@@ -75,7 +75,7 @@ from bitfount.backends.pytorch.types import (
 )
 from bitfount.backends.pytorch.utils import LoggerType, _autodetect_gpu
 from bitfount.backends.pytorch.weight_clipper import _PytorchParamConstraint
-from bitfount.config import BITFOUNT_LOGS_DIR
+from bitfount.config import BITFOUNT_LOGS_DIR, BITFOUNT_OUTPUT_DIR
 from bitfount.data.databunch import BitfountDataBunch
 from bitfount.data.dataloaders import BitfountDataLoader
 from bitfount.data.datasets import _BitfountDataset
@@ -462,10 +462,11 @@ weight-averaging).
                 save_dir=str(BITFOUNT_LOGS_DIR),
             )
         elif logger_config.name == "Neptune":
+            # Neptune logger doesn't have the concept of a save_dir so no need to set
             ml_logger = LoggerType[logger_config.name].value(**logger_config.params)
         else:
             ml_logger = LoggerType[logger_config.name].value(
-                save_dir=logger_config.save_dir or BITFOUNT_LOGS_DIR,
+                save_dir=str(logger_config.save_dir or BITFOUNT_LOGS_DIR),
                 **logger_config.params,
             )
         return ml_logger
@@ -586,6 +587,7 @@ weight-averaging).
                 callbacks=callbacks,
                 gpus=_autodetect_gpu(),  # will be 1 (first GPU) or 0 (CPU)
                 check_val_every_n_epoch=1,
+                default_root_dir=str(BITFOUNT_OUTPUT_DIR),
             )
             return trainer
 
@@ -1244,7 +1246,7 @@ weight-averaging).
         self._pl_trainer.test(model=self, dataloaders=cast(DataLoader, test_dl))
         return np.asarray(self._test_preds), np.asarray(self._test_targets)
 
-    def predict(self, data: BaseSource, **kwargs: Any) -> np.ndarray:
+    def _predict_local(self, data: BaseSource, **kwargs: Any) -> List[np.ndarray]:
         """This method runs inference on the test data, returns predictions.
 
         This is done by calling `test_step` under the hood. Customise this method as you
@@ -1253,23 +1255,23 @@ weight-averaging).
 
         Returns:
             A numpy array containing the prediction values.
-
-        Raises:
-            ValueError: If no data is provided to test with.
         """
-        data.load_data()
-        self._add_datasource_to_schema(data)
-        test_dl = self.databunch.get_test_dataloader(self.batch_size)
-        if isinstance(test_dl, BitfountDataLoader):
-            logger.info(
-                f"Using test portion of dataset for inference - this has "
-                f"{len(test_dl)} record(s)."
-            )
-        else:
-            raise ValueError("No data provided to run inference for the model on.")
+        if data is not None:
+            data.load_data()
+            self._add_datasource_to_schema(data)  # Also sets `self.databunch
+            test_dl = self.databunch.get_test_dataloader(self.batch_size)
+            if isinstance(test_dl, BitfountDataLoader):
+                logger.info(
+                    f"Using test portion of dataset for inference - this has "
+                    f"{len(test_dl)} record(s)."
+                )
+            else:
+                raise ValueError("No test data to infer in the provided datasource.")
 
         self._pl_trainer.test(model=self, dataloaders=cast(DataLoader, test_dl))
-        return cast(np.ndarray, self._test_preds)
+        # Reassuring mypy that `self._test_preds` cannot be None at this point
+        assert self._test_preds is not None  # nosec[assert_used]
+        return self._test_preds
 
     def _reset_test_attrs(self) -> None:
         """Resets test attributes to None."""
