@@ -10,6 +10,7 @@ import numpy as np
 import pytorch_lightning as pl
 from pytorch_lightning.callbacks import Callback
 from pytorch_lightning.callbacks.progress import TQDMProgressBar
+from pytorch_lightning.loggers import TensorBoardLogger
 import torch
 from torch import nn as nn
 from torch.optim.lr_scheduler import _LRScheduler
@@ -23,6 +24,7 @@ from bitfount.backends.pytorch.models.base_models import (
     _OptimizerType,
 )
 from bitfount.backends.pytorch.utils import _autodetect_gpu
+from bitfount.config import BITFOUNT_LOGS_DIR, BITFOUNT_OUTPUT_DIR
 from bitfount.data.databunch import BitfountDataBunch
 from bitfount.data.dataloaders import BitfountDataLoader
 from bitfount.data.datasources.base_source import BaseSource
@@ -380,6 +382,8 @@ class PyTorchBitfountModel(
                 auto_lr_find=True,
                 gpus=_autodetect_gpu(),  # will be 1 (first GPU) or 0 (CPU)
                 callbacks=callbacks,
+                logger=TensorBoardLogger(save_dir=str(BITFOUNT_LOGS_DIR)),
+                default_root_dir=str(BITFOUNT_OUTPUT_DIR),
             )
             return trainer
 
@@ -497,6 +501,14 @@ class PyTorchBitfountModel(
     def test_epoch_end(self, outputs: List[Dict[str, torch.Tensor]]) -> None:  # type: ignore[override] # Reason: see below # noqa: B950
         """Aggregates the predictions and targets from the test set.
 
+        :::caution
+
+        If you are overwriting this method, ensure you set `self._test_preds` to
+        maintain compatibility with `self._predict_local` unless you are overwriting
+        both of them.
+
+        :::
+
         Args:
             outputs: List of outputs from each test step.
         """
@@ -550,32 +562,42 @@ class PyTorchBitfountModel(
         self._test_preds = None
         self._test_targets = None
 
-    def predict(self, data: BaseSource, **kwargs: Any) -> np.ndarray:
+    def _predict_local(self, data: BaseSource, **kwargs: Any) -> List[np.ndarray]:
         """This method runs inference on the test data, returns predictions.
 
         This is done by calling `test_step` under the hood. Customise this method as you
         please but it must return a list of predictions and a list of targets. Note that
         as this is the prediction function, only the predictions are returned.
 
+        :::tip
+
+        Feel free to overwrite this method just so long as you return a numpy array to
+        maintain compatability with the `ModelInference` algorithm - you are not limited
+        to just returning predictions.
+
+        :::
+
         Returns:
             A numpy array containing the prediction values.
-
-        Raises:
-            ValueError: If no data is provided to test with.
         """
-        data.load_data()
-        self._add_datasource_to_schema(datasource=data)
-        test_dl = self.databunch.get_test_dataloader(self.batch_size)
-        if isinstance(test_dl, BitfountDataLoader):
-            logger.info(
-                f"Using test portion of dataset for inference - this has "
-                f"{len(test_dl)} record(s)."
-            )
-        else:
-            raise ValueError("No data provided to run inference for the model on.")
+        if data is not None:
+            data.load_data()
+            self._add_datasource_to_schema(data)  # Also sets `self.databunch
+            test_dl = self.databunch.get_test_dataloader(self.batch_size)
+            if isinstance(test_dl, BitfountDataLoader):
+                logger.info(
+                    f"Using test portion of dataset for inference - this has "
+                    f"{len(test_dl)} record(s)."
+                )
+            else:
+                raise ValueError("No test data to infer in the provided datasource.")
 
         self._pl_trainer.test(model=self, dataloaders=cast(PyTorchDataLoader, test_dl))
-        return cast(np.ndarray, self._test_preds)
+
+        if self._test_preds is not None:
+            return self._test_preds
+
+        raise ValueError("'self._test_preds' was not set by the model after inference.")
 
     def _fit_local(
         self,
