@@ -154,12 +154,14 @@ class DataStructure(BaseDataStructure, _BaseSerializableObjectMixIn):
 
     """
 
-    fields_dict = {
+    fields_dict: ClassVar[T_FIELDS_DICT] = {
         "table": StrDictField(allow_none=True),
         "query": StrDictField(allow_none=True),
         "schema_types_override": fields.Dict(allow_none=True),
         "target": fields.Raw(allow_none=True),
-        "ignore_cols": fields.List(fields.Str(), allow_none=True),
+        # `ignore_cols` is intentionally not serialised because it can be reconstructed
+        # from the `selected_cols`. Furthermore, when it comes to deserialisation, the
+        # datastructure can only accept one of these 2 arguments
         "selected_cols": fields.List(fields.Str(), allow_none=True),
         "loss_weights_col": fields.Str(allow_none=True),
         "multihead_col": fields.Str(allow_none=True),
@@ -180,19 +182,18 @@ class DataStructure(BaseDataStructure, _BaseSerializableObjectMixIn):
             allow_none=True,
         ),
         "auto_convert_grayscale_images": fields.Boolean(),
-    }  # type: ClassVar[T_FIELDS_DICT]
-    nested_fields = {}  # type: ClassVar[T_NESTED_FIELDS]
+    }
+    nested_fields: ClassVar[T_NESTED_FIELDS] = {}
     table: Optional[Union[str, Mapping[str, str]]] = None
     query: Optional[Union[str, Mapping[str, str]]] = None
     schema_types_override: Optional[
         Union[SchemaOverrideMapping, Mapping[str, SchemaOverrideMapping]]
     ] = None
     target: Optional[Union[str, List[str]]] = None
-    # Mypy errors ignored below. For more details see `config_schemas.py`
-    ignore_cols: List[str] = desert.field(  # type: ignore[assignment] # Reason: above
+    ignore_cols: List[str] = desert.field(
         fields.List(fields.String()), default_factory=list
     )
-    selected_cols: List[str] = desert.field(  # type: ignore[assignment] # Reason: above
+    selected_cols: List[str] = desert.field(
         fields.List(fields.String()), default_factory=list
     )
     data_splitter: Optional[DatasetSplitter] = None
@@ -323,6 +324,7 @@ class DataStructure(BaseDataStructure, _BaseSerializableObjectMixIn):
             image_cols=assign.image_cols,
             batch_transforms=transform.batch,
             dataset_transforms=transform.dataset,
+            auto_convert_grayscale_images=transform.auto_convert_grayscale_images,
         )
 
     def get_table_name(self, pod_identifier: Optional[str] = None) -> str:
