@@ -14,6 +14,18 @@ class DataStructureError(BitfountError):
     pass
 
 
+class DataSourceError(BitfountError):
+    """Errors related to Datasource."""
+
+    pass
+
+
+class DatasetSplitterError(BitfountError):
+    """Errors related to DatasetSplitter."""
+
+    pass
+
+
 class DatabaseSchemaNotFoundError(BitfountError):
     """Raised when a specified database schema is not found."""
 
@@ -26,11 +38,8 @@ class DatabaseMissingTableError(BitfountError):
     pass
 
 
-class DataNotSplitError(BitfountError):
-    """Raised when a particular data split is requested which does not exist yet.
-
-    This is usually raised because `split_data` has not been called yet.
-    """
+class DatabaseUnsupportedQueryError(BitfountError):
+    """Raised when an unsupported database query is provided."""
 
     pass
 
@@ -51,3 +60,13 @@ class DuplicateColumnError(BitfountError):
     """
 
     pass
+
+
+class ExcelSourceError(BitfountError):
+    """Error for ExcelSources.
+
+    We raise this if trying to get the types of the
+    columns with no table name in a multi-table datasource.
+    """
+
+    pass
