@@ -29,7 +29,10 @@ import yaml
 
 import bitfount
 from bitfount.data import BaseSource
-from bitfount.data.datasources.base_source import MultiTableSource
+from bitfount.data.datasources.base_source import (
+    FileSystemIterableSource,
+    MultiTableSource,
+)
 from bitfount.data.exceptions import BitfountSchemaError, DataSourceError
 from bitfount.data.types import (
     CategoricalRecord,
@@ -194,6 +197,30 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
 
         return {k: v for k, v in semantic_types.items() if len(v) > 0}
 
+    def decode_categorical(self, feature: str, value: int) -> Any:
+        """Decode label corresponding to a categorical feature in the schema.
+
+        Args:
+            feature: The name of the feature.
+            value: The encoded value.
+
+        Returns:
+            The decoded feature value.
+
+        Raises:
+            ValueError: If the feature cannot be found in the schema.
+            ValueError: If the label cannot be found in the feature encoder.
+        """
+        if feature not in self.features["categorical"]:
+            raise ValueError(
+                f"Could not find {feature} in categorical features of the schema."
+            )
+        for k, v in self.features["categorical"][feature].encoder.classes.items():
+            if v == value:
+                return k
+
+        raise ValueError(f"Could not find {value} in {feature}.")
+
     def _add_categorical_feature(
         self,
         name: str,
@@ -326,6 +353,15 @@ class TableSchema(_TableSchemaMarshmallowMixIn):
                             dtype=dtype,
                             description=descriptions.get(feature_name),
                         )
+
+                    # If the datasource is a FileSystemIterableSource and `fast_load` is
+                    # True, we won't iterate over the images to get their shapes and
+                    # formats, etc.
+                    if (
+                        isinstance(datasource, FileSystemIterableSource)
+                        and datasource.fast_load
+                    ) or datasource.iterable:
+                        continue
                     record = self.features["image"][feature_name]
                     for img in datasource.get_column(
                         feature_name, table_name=self.name
