@@ -1,21 +1,13 @@
-"""Manages the federated communication and training of models."""
+"""Manages the federated communication and training of models.
+
+Federated algorithm plugins can also be imported from this package.
+"""
 from typing import List
 
 from bitfount.federated.aggregators.aggregator import Aggregator
 from bitfount.federated.aggregators.secure import SecureAggregator
-from bitfount.federated.algorithms.column_avg import ColumnAverage
-from bitfount.federated.algorithms.compute_intersection_rsa import (
-    ComputeIntersectionRSA,
-)
-from bitfount.federated.algorithms.model_algorithms.evaluate import ModelEvaluation
-from bitfount.federated.algorithms.model_algorithms.federated_training import (
-    FederatedModelTraining,
-)
-from bitfount.federated.algorithms.model_algorithms.train_and_evaluate import (
-    ModelTrainingAndEvaluation,
-)
-from bitfount.federated.algorithms.private_sql_query import PrivateSqlQuery
-from bitfount.federated.algorithms.sql_query import SqlQuery
+import bitfount.federated.algorithms as algorithms
+from bitfount.federated.algorithms import *  # noqa: F401, F403
 from bitfount.federated.authorisation_checkers import IdentityVerificationMethod
 from bitfount.federated.early_stopping import FederatedEarlyStopping
 from bitfount.federated.exceptions import (
@@ -30,19 +22,19 @@ from bitfount.federated.exceptions import (
     PodNameError,
     PodRegistrationError,
     PrivateSqlError,
+    PSIMultiTableError,
+    PSIUnsupportedDataSourceError,
     SecureShareError,
 )
 from bitfount.federated.helper import combine_pod_schemas
+from bitfount.federated.keys_setup import RSAKeyPair
+from bitfount.federated.logging import _get_federated_logger
 from bitfount.federated.model_reference import BitfountModelReference
 from bitfount.federated.modeller import _Modeller
 from bitfount.federated.pod import Pod
-from bitfount.federated.pod_keys_setup import PodKeys
 from bitfount.federated.privacy.differential import DPModellerConfig, DPPodConfig
-from bitfount.federated.protocols.model_protocols.federated_averaging import (
-    FederatedAveraging,
-)
-from bitfount.federated.protocols.psi import PrivateSetIntersection
-from bitfount.federated.protocols.results_only import ResultsOnly
+import bitfount.federated.protocols as protocols
+from bitfount.federated.protocols import *  # noqa: F401, F403
 from bitfount.federated.roles import Role
 from bitfount.federated.secure import SecureShare
 from bitfount.federated.shim import BackendTensorShim
@@ -53,6 +45,8 @@ from bitfount.federated.transport.config import (
 )
 from bitfount.federated.types import AggregatorType, AlgorithmType, ProtocolType
 
+_logger = _get_federated_logger(__name__)
+
 __all__: List[str] = [
     "Aggregator",
     "AggregatorError",
@@ -61,43 +55,40 @@ __all__: List[str] = [
     "BackendTensorShim",
     "BitfountModelReference",
     "BitfountTaskStartError",
-    "ColumnAverage",
     "DPModellerConfig",
     "DPPodConfig",
     "DecryptError",
     "EncryptError",
     "EncryptionError",
-    "FederatedAveraging",
     "FederatedEarlyStopping",
-    "FederatedModelTraining",
     "IdentityVerificationMethod",
     "MAXIMUM_GRPC_MESSAGE_SIZE_BYTES",
     "MessageHandlerNotFoundError",
     "MessageRetrievalError",
     "MessageServiceConfig",
-    "ModelEvaluation",
-    "ModelTrainingAndEvaluation",
     "_Modeller",
     "Pod",
     "PodConnectFailedError",
-    "PodKeys",
+    "RSAKeyPair",
     "PodNameError",
     "PodRegistrationError",
-    "PrivateSetIntersection",
     "PrivateSqlError",
-    "PrivateSqlQuery",
     "ProtocolType",
     "PRODUCTION_MESSAGE_SERVICE_URL",
-    "ResultsOnly",
+    "PSIMultiTableError",
+    "PSIUnsupportedDataSourceError",
     "Role",
-    "ComputeIntersectionRSA",
     "SecureAggregator",
     "SecureShare",
     "SecureShareError",
-    "SqlQuery",
     "combine_pod_schemas",
 ]
 
+# Protocols and algorithms are imported from their own respective subpackages because
+# of how we handle plugins for these components.
+__all__.extend(algorithms.__all__)
+__all__.extend(protocols.__all__)
+
 # See top level `__init__.py` for an explanation
 __pdoc__ = {}
 for _obj in __all__:
