@@ -52,14 +52,14 @@ class _BaseAlgorithm(ABC):
         return decorate(func, federated_exception)
 
 
-class _BaseModellerAlgorithm(_BaseAlgorithm, ABC):
+class BaseModellerAlgorithm(_BaseAlgorithm, ABC):
     """Modeller side of the algorithm."""
 
     def __init__(self, **kwargs: Any):
         super().__init__(**kwargs)
 
 
-class _BaseWorkerAlgorithm(_BaseAlgorithm, ABC):
+class BaseWorkerAlgorithm(_BaseAlgorithm, ABC):
     """Worker side of the algorithm."""
 
     def __init__(self, **kwargs: Any):
@@ -98,12 +98,12 @@ class _BaseWorkerAlgorithm(_BaseAlgorithm, ABC):
 
 
 # The mutable underlying dict that holds the registry information
-_registry: Dict[str, Type[_BaseAlgorithmFactory]] = {}
+_registry: Dict[str, Type[BaseAlgorithmFactory]] = {}
 # The read-only version of the registry that is allowed to be imported
-registry: Mapping[str, Type[_BaseAlgorithmFactory]] = MappingProxyType(_registry)
+registry: Mapping[str, Type[BaseAlgorithmFactory]] = MappingProxyType(_registry)
 
 
-class _BaseAlgorithmFactory(ABC, _RolesMixIn, _BaseSerializableObjectMixIn):
+class BaseAlgorithmFactory(ABC, _RolesMixIn, _BaseSerializableObjectMixIn):
     """Base algorithm factory from which all other algorithms must inherit.
 
     Attributes:
@@ -114,7 +114,11 @@ class _BaseAlgorithmFactory(ABC, _RolesMixIn, _BaseSerializableObjectMixIn):
     nested_fields: ClassVar[T_NESTED_FIELDS] = {}
 
     def __init__(self, **kwargs: Any):
-        self.class_name = AlgorithmType[type(self).__name__].value
+        try:
+            self.class_name = AlgorithmType[type(self).__name__].value
+        except KeyError:
+            # Check if the algorithm is a plug-in
+            self.class_name = type(self).__name__
         super().__init__(**kwargs)
 
     @classmethod
