@@ -1,14 +1,11 @@
 """Algorithm to train and evaluate a model on remote data."""
 from __future__ import annotations
 
-from typing import Any, Dict, List, Type, cast
+from typing import Any, Dict, Mapping, cast
 
-from marshmallow import Schema as MarshmallowSchema
-from marshmallow import fields, post_load
 import numpy as np
 
-from bitfount.data.datasource import DataSource
-from bitfount.federated.algorithms.base import _BaseAlgorithmFactory
+from bitfount.data.datasources.base_source import BaseSource
 from bitfount.federated.algorithms.model_algorithms.base import (
     _BaseModelAlgorithmFactory,
     _BaseModellerModelAlgorithm,
@@ -17,6 +14,7 @@ from bitfount.federated.algorithms.model_algorithms.base import (
 from bitfount.federated.logging import _get_federated_logger
 from bitfount.hub.api import BitfountHub
 from bitfount.metrics import MetricCollection
+from bitfount.utils import delegates
 
 logger = _get_federated_logger(__name__)
 
@@ -24,40 +22,42 @@ logger = _get_federated_logger(__name__)
 class _ModellerSide(_BaseModellerModelAlgorithm):
     """Modeller side of the ModelTrainingAndEvaluation algorithm."""
 
-    def run(self, results: List[Dict[str, float]]) -> List[Dict[str, float]]:
+    def run(
+        self, results: Mapping[str, Dict[str, float]]
+    ) -> Dict[str, Dict[str, float]]:
         """Simply returns results."""
-        return results
+        return dict(results)
 
 
 class _WorkerSide(_BaseWorkerModelAlgorithm):
     """Worker side of the ModelTrainingAndEvaluation algorithm."""
 
-    def run(self, data: DataSource) -> Dict[str, float]:
+    def run(self, data: BaseSource) -> Dict[str, float]:
         """Runs training and evaluation and returns metrics."""
         self.model.fit(data)
-        preds, targs = self.model.evaluate()
+        preds, target = self.model.evaluate()
         # TODO: [BIT-1604] Remove these cast statements once they become superfluous.
         preds = cast(np.ndarray, preds)
-        targs = cast(np.ndarray, targs)
+        target = cast(np.ndarray, target)
         m = MetricCollection.create_from_model(self.model)
-        return m.compute(targs, preds)
+        return m.compute(target, preds)
 
 
+@delegates()
 class ModelTrainingAndEvaluation(_BaseModelAlgorithmFactory):
     """Algorithm for training a model, evaluating it and returning metrics.
 
-    Args:
-        model: The model to train and evaluate on remote data.
-
-    Attributes:
-        name: The name of the algorithm.
-        model: The model to train and evaluate on remote data.
-
     :::note
 
     The metrics cannot currently be specified by the user.
 
     :::
+
+    Args:
+        model: The model to train and evaluate on remote data.
+
+    Attributes:
+        model: The model to train and evaluate on remote data.
     """
 
     def modeller(self, **kwargs: Any) -> _ModellerSide:
@@ -73,25 +73,3 @@ class ModelTrainingAndEvaluation(_BaseModelAlgorithmFactory):
         """
         model = self._get_model_from_reference(hub=hub)
         return _WorkerSide(model=model, **kwargs)
-
-    @staticmethod
-    def get_schema(
-        model_schema: Type[MarshmallowSchema], **kwargs: Any
-    ) -> Type[MarshmallowSchema]:
-        """Returns the schema for ModelTrainingAndEvaluation.
-
-        Args:
-            model_schema: The schema for the underlying model.
-        """
-
-        class Schema(_BaseAlgorithmFactory._Schema):
-
-            model = fields.Nested(model_schema)
-
-            @post_load
-            def recreate_factory(
-                self, data: dict, **_kwargs: Any
-            ) -> ModelTrainingAndEvaluation:
-                return ModelTrainingAndEvaluation(**data)
-
-        return Schema
