@@ -5,7 +5,18 @@ import asyncio
 from dataclasses import dataclass
 from datetime import datetime, timezone
 from enum import Enum
-from typing import Callable, Final, List, Mapping, Protocol, Tuple, Type, TypeVar, Union
+from typing import (
+    Callable,
+    Final,
+    List,
+    Mapping,
+    Optional,
+    Protocol,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
 import requests
@@ -28,10 +39,10 @@ from bitfount.federated.types import (
     _TaskRequestMessageGenerator,
 )
 from bitfount.hub.api import BitfountAM
-from bitfount.hub.authentication_flow import (
+from bitfount.hub.authentication_flow import _get_auth_environment
+from bitfount.hub.authentication_handlers import (
     _AUTHORIZATION_PENDING_ERROR,
     _SLOW_DOWN_ERROR,
-    _get_auth_environment,
 )
 from bitfount.hub.types import (
     _DeviceAccessTokenFailResponseJSON,
@@ -139,6 +150,7 @@ def _make_encrypted_task_request_message(
     pod_identifiers: List[str],
     aes_key: bytes,
     pod_public_key: RSAPublicKey,
+    project_id: Optional[str],
 ) -> bytes:
     """Common function for generating messages with _EncryptedTaskRequest contents."""
     # Create task request
@@ -151,9 +163,7 @@ def _make_encrypted_task_request_message(
 
     # Create outer message
     message = _TaskRequestMessage(
-        serialized_protocol,
-        auth_type.value,
-        packed_tr.serialize(),
+        serialized_protocol, auth_type.value, packed_tr.serialize(), project_id
     )
     return message.serialize()
 
@@ -223,6 +233,7 @@ class _LocalAuthorisation(_AuthorisationChecker[_EncryptedTaskRequest]):
         pod_identifiers: List[str],
         aes_key: bytes,
         pod_public_key: RSAPublicKey,
+        project_id: Optional[str],
     ) -> bytes:
         return _make_encrypted_task_request_message(
             IdentityVerificationMethod.LOCAL,
@@ -230,6 +241,7 @@ class _LocalAuthorisation(_AuthorisationChecker[_EncryptedTaskRequest]):
             pod_identifiers,
             aes_key,
             pod_public_key,
+            project_id,
         )
 
     @classmethod
@@ -282,6 +294,7 @@ class _SignatureBasedAuthorisation(_AuthorisationChecker[_SignedEncryptedTaskReq
             pod_identifiers: List[str],
             aes_key: bytes,
             pod_public_key: RSAPublicKey,
+            project_id: Optional[str] = None,
         ) -> bytes:
             tr = _TaskRequest(serialized_protocol, pod_identifiers, aes_key)
 
@@ -293,6 +306,7 @@ class _SignatureBasedAuthorisation(_AuthorisationChecker[_SignedEncryptedTaskReq
                 serialized_protocol,
                 IdentityVerificationMethod.KEYS.value,
                 packed_tr.serialize(),
+                project_id,
             )
             return message.serialize()
 
@@ -337,9 +351,7 @@ class _SignatureBasedAuthorisation(_AuthorisationChecker[_SignedEncryptedTaskReq
         """Check that access is allowed for the authorised user and request."""
         return self.access_manager.check_signature_based_access_request(
             unsigned_task=self.encrypted_task_request,
-            signed_task=_SignedEncryptedTaskRequest(
-                self.encrypted_task_request, self.signature
-            ).serialize(),
+            task_signature=self.signature,
             pod_identifier=self.pod_response_message.pod_identifier,
             serialized_protocol=self.serialized_protocol,
             modeller_name=self.modeller_name,
@@ -374,6 +386,7 @@ class _SAMLAuthorisation(_AuthorisationChecker[_EncryptedTaskRequest]):
         pod_identifiers: List[str],
         aes_key: bytes,
         pod_public_key: RSAPublicKey,
+        project_id: Optional[str],
     ) -> bytes:
         return _make_encrypted_task_request_message(
             IdentityVerificationMethod.SAML,
@@ -381,6 +394,7 @@ class _SAMLAuthorisation(_AuthorisationChecker[_EncryptedTaskRequest]):
             pod_identifiers,
             aes_key,
             pod_public_key,
+            project_id,
         )
 
     @classmethod
@@ -479,6 +493,7 @@ class _OIDCAuthorisationCode(_AuthorisationChecker[_EncryptedTaskRequest]):
         pod_identifiers: List[str],
         aes_key: bytes,
         pod_public_key: RSAPublicKey,
+        project_id: Optional[str],
     ) -> bytes:
         return _make_encrypted_task_request_message(
             IdentityVerificationMethod.OIDC_ACF_PKCE,
@@ -486,6 +501,7 @@ class _OIDCAuthorisationCode(_AuthorisationChecker[_EncryptedTaskRequest]):
             pod_identifiers,
             aes_key,
             pod_public_key,
+            project_id,
         )
 
     @classmethod
@@ -617,6 +633,7 @@ class _OIDCDeviceCode(_AuthorisationChecker[_EncryptedTaskRequest]):
         pod_identifiers: List[str],
         aes_key: bytes,
         pod_public_key: RSAPublicKey,
+        project_id: Optional[str],
     ) -> bytes:
         return _make_encrypted_task_request_message(
             IdentityVerificationMethod.OIDC_DEVICE_CODE,
@@ -624,6 +641,7 @@ class _OIDCDeviceCode(_AuthorisationChecker[_EncryptedTaskRequest]):
             pod_identifiers,
             aes_key,
             pod_public_key,
+            project_id,
         )
 
     @classmethod
