@@ -2,8 +2,10 @@
 import asyncio
 from contextlib import nullcontext
 from dataclasses import asdict
+import hashlib
 import os
 from pathlib import Path
+import sqlite3
 import threading
 from typing import (
     Any,
@@ -21,6 +23,8 @@ from typing import (
 )
 
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
+import numpy as np
+import pandas as pd
 from requests import HTTPError, RequestException
 import yaml
 
@@ -44,9 +48,9 @@ from bitfount.federated.helper import (
     _check_and_update_pod_ids,
     _create_and_connect_pod_mailbox,
 )
+from bitfount.federated.keys_setup import RSAKeyPair, _get_pod_keys
 from bitfount.federated.logging import _get_federated_logger
 from bitfount.federated.monitoring import task_monitor_context
-from bitfount.federated.pod_keys_setup import PodKeys, _get_pod_keys
 from bitfount.federated.pod_response_message import _PodResponseMessage
 from bitfount.federated.pod_vitals import _PodVitals, _PodVitalsHandler
 from bitfount.federated.privacy.differential import DPPodConfig
@@ -67,13 +71,14 @@ from bitfount.federated.transport.worker_transport import (
 )
 from bitfount.federated.types import (
     AggregatorType,
+    SerializedAlgorithm,
     SerializedProtocol,
     _PodResponseType,
 )
-from bitfount.federated.utils import _keys_exist
 from bitfount.federated.worker import _Worker
 from bitfount.hub.api import BitfountAM, BitfountHub, PodPublicMetadata
-from bitfount.hub.authentication_flow import _DEFAULT_USERNAME, _get_auth_environment
+from bitfount.hub.authentication_flow import _get_auth_environment
+from bitfount.hub.authentication_handlers import _DEFAULT_USERNAME
 from bitfount.hub.exceptions import SchemaUploadError
 from bitfount.hub.helper import (
     _create_access_manager,
@@ -81,8 +86,10 @@ from bitfount.hub.helper import (
     _get_pod_public_keys,
 )
 from bitfount.runners.config_schemas import (
+    JWT,
     POD_NAME_REGEX,
     AccessManagerConfig,
+    APIKeys,
     PodConfig,
     PodDataConfig,
     PodDetailsConfig,
@@ -92,7 +99,7 @@ from bitfount.transformations.dataset_operations import (
     NormalizeDataTransformation,
 )
 from bitfount.transformations.processor import TransformationProcessor
-from bitfount.utils import _handle_fatal_error, _is_notebook
+from bitfount.utils import _handle_fatal_error, is_notebook
 
 logger = _get_federated_logger(__name__)
 
@@ -153,19 +160,30 @@ class Pod:
         username: Username of the user who is registering the pod. Defaults to None.
         data_config: Configuration for the data. Defaults to None.
         schema: Schema for the data. This can be a `BitfountSchema` object or a Path
-            to a serialized `BitfountSchema`. This will generated automatically if not
-            provided. Defaults to None.
+            to a serialized `BitfountSchema`. This will be generated automatically
+            if not provided. Defaults to None.
         pod_details_config: Configuration for the pod details. Defaults to None.
-        bitfount_hub: Bitfount Hub to register the pod with. Defaults to None.
-        ms_config: Configuration for the message service. Defaults to None.
+        hub: Bitfount Hub to register the pod with. Defaults to None.
+        message_service: Configuration for the message service. Defaults to None.
         access_manager: Access manager to use for checking access. Defaults to None.
         pod_keys: Keys for the pod. Defaults to None.
         approved_pods: List of other pod identifiers this pod is happy
             to share a training task with. Required if the protocol uses the
             `SecureAggregator` aggregator.
-        pod_dp: Differential privacy configuration for the pod. Defaults to None.
+        differential_privacy: Differential privacy configuration for the pod.
+            Defaults to None.
+        pod_db: Whether the results should be stored in a database. Defaults to False.
+            If argument is set to True, then a SQLite database will be created for the
+            pod in order to enable results storage for protocols that return them.
+            It also keeps track of the pod datatpoints so any repeat task is ran
+            only on new datapoints.
+        show_datapoints_with_results_in_db: Whether the original datapoints should be
+            included in the results database. Defaults to True. This argument is ignored
+            if pod_db argument is set to False.
         update_schema: Whether the schema needs to be re-generated even if provided.
             Defaults to False.
+        secrets: Secrets for authenticating with Bitfount services.
+            If not provided then an interactive flow will trigger for authentication.
 
     Attributes:
         data: Data that the pod encapsulates.
@@ -194,13 +212,15 @@ class Pod:
         hub: Optional[BitfountHub] = None,
         message_service: Optional[MessageServiceConfig] = None,
         access_manager: Optional[BitfountAM] = None,
-        pod_keys: Optional[PodKeys] = None,
+        pod_keys: Optional[RSAKeyPair] = None,
         approved_pods: Optional[List[str]] = None,
         differential_privacy: Optional[DPPodConfig] = None,
+        pod_db: bool = False,
+        show_datapoints_with_results_in_db: bool = True,
         update_schema: bool = False,
+        secrets: Optional[Union[APIKeys, JWT]] = None,
     ):
         self.name = name
-
         if not data_config:
             data_config = PodDataConfig()
 
@@ -221,6 +241,7 @@ class Pod:
             else MessageServiceConfig(),
             approved_pods=approved_pods,
             update_schema=update_schema,
+            secrets=secrets,
         )
         # Fast-fail if BaseSource has not been initialised
         if not datasource.is_initialised:
@@ -238,7 +259,6 @@ class Pod:
 
         # If the schema was a path, it should be loaded by here, so we can cast:
         schema = cast(Optional[BitfountSchema], schema)
-
         if self.datasource.multi_table and schema is not None:
             # check if all table names are in the schema
             schema_tables = [table.name for table in schema.tables]
@@ -258,11 +278,10 @@ class Pod:
         elif schema is not None and self.name not in [
             table.name for table in schema.tables
         ]:
-            logger.info(
+            logger.warning(
                 "Provided schema table name does not "
-                "match to pod name, re-generating schema."
+                "match to pod name, you may need to regenerate the schema."
             )
-            update_schema = True
 
         if update_schema is True or schema is None:
             self.schema = self._setup_schema(schema=schema, data_config=data_config)
@@ -273,8 +292,18 @@ class Pod:
         self.public_metadata = self._get_public_metadata(
             pod_details_config, self.schema
         )
+        self.pod_db = pod_db
+        if self.pod_db:
+            self._update_pod_db()
+        self.show_datapoints_with_results_in_db = (
+            show_datapoints_with_results_in_db if self.pod_db else False
+        )
 
-        self._hub = hub if hub is not None else _create_bitfounthub(username=username)
+        self._hub = (
+            hub
+            if hub is not None
+            else _create_bitfounthub(username=username, secrets=secrets)
+        )
         self._session = self._hub.session
 
         self._access_manager = (
@@ -295,7 +324,6 @@ class Pod:
         self.approved_pods = approved_pods
         self._pod_dp = differential_privacy
         self._pod_vitals = _PodVitals()
-
         # Connecting the pod to the message service must happen AFTER registering
         # it on the hub as the message service uses hub information to verify that
         # the relevant message queue is available.
@@ -359,7 +387,6 @@ class Pod:
             force_stypes=data_config.force_stypes,
         )
 
-        # Auto-tidy if specified and datasource is not multi-table
         if data_config.auto_tidy and self.datasource.multi_table:
             logger.warning("Can't autotidy multi-table data.")
         elif data_config.auto_tidy:
@@ -385,6 +412,103 @@ class Pod:
         schema.freeze()
         return schema
 
+    def _update_pod_db(self) -> None:
+        """Creates and updates the pod database.
+
+        This is a static database on the pod with the datapoint hashes so we only
+        compute them once. For each datapoint row in the datasource, a hash value
+        is computed. Then the data from the datasource, together with the hash value,
+        are written to the database.
+
+        :::note
+
+        Only works for single-table, non-iterable datasources.
+
+        :::
+
+        Raises:
+            ValueError: If there are clashing column names in the datasource
+                and the pod database.
+        """
+        if self.datasource.multi_table or self.datasource.iterable:
+            # TODO: [BIT-2486] - Look into supporting multi-table datasources.
+            logger.warning(
+                "Pod database only supported with single-table, "
+                "non-iterable datasources."
+            )
+            self.pod_db = False
+        else:
+            con = sqlite3.connect(f"{self.name}.db")
+            cur = con.cursor()
+            cur.execute(
+                """CREATE TABLE IF NOT EXISTS "datasource" ('rowID' INTEGER PRIMARY KEY)"""  # noqa: B950
+            )
+            con.commit()
+
+            # get the existing datasource data
+            self.datasource.load_data()
+            new_data = self.datasource.data
+            if "datapoint_hash" in new_data.columns:
+                raise ValueError(
+                    "`datapoint_hash` not supported as column name in the datasource."
+                )
+            # as placeholder column for the hash
+            new_data["datapoint_hash"] = np.nan
+            # change type to object so it will match the db type
+            new_data["datapoint_hash"] = new_data["datapoint_hash"].astype("object")
+
+            # sqlite transforms bool values to int, so we need to make sure that
+            # they are the same in the df so the hashes match
+            bool_cols = [col for col in new_data.columns if new_data[col].dtype == bool]
+            # replace bools by their int value, as it will be done by
+            # sqlite in the db anyway
+            new_data[bool_cols] *= 1
+            # read the db data for the datasource
+            existing_data = pd.read_sql('SELECT * FROM "datasource"', con)
+            existing_cols_without_index = set(existing_data.columns)
+            existing_cols_without_index.remove("rowID")
+            # check if df is empty or if columns all columns are the same,
+            # if not all the hashes will have to be recomputed
+            if (
+                not existing_data.empty
+                and set(new_data.columns) == existing_cols_without_index
+            ):
+                data = pd.merge(
+                    new_data.drop(columns=["datapoint_hash"]),
+                    existing_data.drop(columns=["datapoint_hash", "rowID"]),
+                    how="outer",
+                    indicator=True,
+                ).loc[lambda x: x["_merge"] != "both"]
+                data.drop(columns=["_merge"], inplace=True)
+                data.drop_duplicates()
+                hashed_list = []
+                for _, row in data.iterrows():
+                    hashed_list.append(
+                        hashlib.sha256(str(row).encode("utf-8")).hexdigest()
+                    )
+                data["datapoint_hash"] = hashed_list
+                data.to_sql("datasource", con=con, if_exists="append", index=False)
+            else:
+
+                # replace table if columns are mismatched
+                cur.execute("DROP TABLE 'datasource'")
+                cur.execute(
+                    """CREATE TABLE "datasource" ('rowID' INTEGER PRIMARY KEY)"""
+                )
+                for col in new_data.columns:
+                    cur.execute(
+                        f"ALTER TABLE 'datasource' ADD COLUMN '{col}' {new_data[col].dtype}"  # noqa: B950
+                    )
+
+                hashed_list = []
+                for _, row in new_data.iterrows():
+                    hashed_list.append(
+                        hashlib.sha256(str(row).encode("utf-8")).hexdigest()
+                    )
+                data = new_data
+                data["datapoint_hash"] = hashed_list
+                data.to_sql("datasource", con=con, if_exists="append", index=False)
+
     def _get_default_pod_details_config(self) -> PodDetailsConfig:
         """Get default pod details config."""
         return PodDetailsConfig(display_name=self.name, description=self.name)
@@ -401,7 +525,7 @@ class Pod:
         )
 
     def _get_default_pod_keys(
-        self, pod_keys: Optional[PodKeys]
+        self, pod_keys: Optional[RSAKeyPair]
     ) -> Tuple[RSAPrivateKey, RSAPublicKey]:
         """Get default pod keys."""
         if pod_keys is None:
@@ -547,6 +671,9 @@ class Pod:
         task_request_message: _TaskRequestMessage = _TaskRequestMessage.deserialize(
             message.body
         )
+        # TODO: [BIT-2486] There should be check to see if the
+        #  task request contains a project id, and allow
+        #  for pod_db only in that case.
         auth_type: IdentityVerificationMethod = check_identity_verification_method(
             task_request_message.auth_type
         )
@@ -626,6 +753,8 @@ class Pod:
         )
 
         logger.debug("Creating worker.")
+        # TODO: [BIT-2486] Add the project id to the worker to ensure
+        #  results are written to the correct project db.
         worker = _Worker(
             datasource=self.datasource,
             mailbox=worker_mailbox,
@@ -635,6 +764,8 @@ class Pod:
             pod_dp=self._pod_dp,
             pod_identifier=self.pod_identifier,
             serialized_protocol=task_request.serialized_protocol,
+            pod_db=self.pod_db,
+            show_datapoints_in_results_db=self.show_datapoints_with_results_in_db,
         )
 
         # If interacting with an older modeller version then task_id won't be supplied
@@ -692,9 +823,18 @@ class Pod:
             task_request_message, self.private_key
         )
         serialized_protocol = task_request.serialized_protocol
-        # remove schema since it is the largest task element.
-        if _keys_exist(serialized_protocol, "algorithm", "model", "schema"):  # type: ignore[arg-type] # Reason: a typed dict is still a dict. # noqa: B950
-            serialized_protocol["algorithm"]["model"].pop("schema")  # type: ignore[typeddict-item] # Reason: all models have a schema associated with them # noqa: B950
+        # Remove schema to reduce latency when checking access with the Access Manager
+        # since it is the largest task element.
+        algorithm = serialized_protocol["algorithm"]
+        if not isinstance(serialized_protocol["algorithm"], list):
+            algorithm = [cast(SerializedAlgorithm, algorithm)]
+
+        algorithm = cast(List[SerializedAlgorithm], algorithm)
+        for algo in algorithm:
+            try:
+                algo["model"].pop("schema", None)
+            except KeyError:
+                pass
 
         pod_response_message = _PodResponseMessage(
             modeller_name=sender,
@@ -781,7 +921,7 @@ class Pod:
 
     def _run_pod_heartbeat_task(self, stop_event: threading.Event) -> None:
         """Makes 10-second interval pod heartbeats to the hub."""
-        if _is_notebook():
+        if is_notebook():
             # We need to create a new event loop here for jupyter
             # As it's run in a new thread and can't be patched by nest_asyncio
             asyncio.set_event_loop(asyncio.new_event_loop())
@@ -815,7 +955,7 @@ class Pod:
     def _run_pod_vitals_server(self) -> Optional[_PodVitalsHandler]:
         """Create _PodVitalsHandelr and run _PodVitals webserver."""
         # Check that we have not initialized the Pod from a notebook
-        if not _is_notebook():
+        if not is_notebook():
             # Setup pod vitals webserver
             vitals_handler = _PodVitalsHandler(self._pod_vitals)
             logger.debug("Starting Pod Vitals interface...")
@@ -857,6 +997,8 @@ class Pod:
 
         # Start pod listening for messages
         logger.info("Pod started... press Ctrl+C to stop")
+        # Mark as ready in the pod vitals, even if we're not using the handler
+        self._pod_vitals.mark_pod_ready()
         try:
             await self._mailbox.listen_indefinitely()
         finally:
