@@ -92,14 +92,14 @@ class DPModellerConfig:
     noise_multiplier: float = 0.4
     alphas: List[float] = field(default_factory=lambda: _DEFAULT_ALPHAS)
     delta: float = _DEFAULT_DELTA
-    loss_reduction: Literal["mean", "sum"] = desert.field(  # type: ignore[assignment] # Reason: desert.field is intentional # noqa: B950
+    loss_reduction: Literal["mean", "sum"] = desert.field(
         marshmallow.fields.String(validate=OneOf(_ALLOWED_LOSS_REDUCTIONS)),
         default="mean",
     )
 
     # Other options
     auto_fix: bool = True
-    fields_dict = {
+    fields_dict: ClassVar[_StrAnyDict] = {
         "epsilon": fields.Float(),
         "max_grad_norm": M_Union([fields.Float(), fields.List(fields.Float())]),
         "noise_multiplier": fields.Float(),
@@ -107,8 +107,8 @@ class DPModellerConfig:
         "delta": fields.Float(),
         "loss_reduction": fields.String(),
         "auto_fix": fields.Bool(),
-    }  # type:  _StrAnyDict
-    nested_fields = {}  # type:Dict[str, _StrAnyDict]
+    }
+    nested_fields: ClassVar[Dict[str, _StrAnyDict]] = {}
 
     def __post_init__(self) -> None:
         # Validate loss_reduction
