@@ -2,13 +2,16 @@
 from __future__ import annotations
 
 import asyncio
+from asyncio.futures import Future as AsyncFuture
 import atexit
 from collections.abc import Iterable
+from concurrent.futures import Future as ConcurrentFuture
 from concurrent.futures import ThreadPoolExecutor
 from dataclasses import dataclass
 from functools import wraps
 import inspect
 import threading
+from threading import Lock as ThreadingLock
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -31,6 +34,7 @@ import warnings
 import weakref
 
 from bitfount.federated.exceptions import DecryptError
+from bitfount.federated.logging import _get_federated_logger
 from bitfount.federated.transport.message_service import (
     _BitfountMessage,
     _BitfountMessageType,
@@ -38,13 +42,7 @@ from bitfount.federated.transport.message_service import (
 from bitfount.utils.concurrency_utils import await_event_with_stop
 
 if TYPE_CHECKING:
-    from bitfount.federated.transport.base_transport import _BaseMailbox, Handler
-
-from asyncio.futures import Future as AsyncFuture
-from concurrent.futures import Future as ConcurrentFuture
-from threading import Lock as ThreadingLock
-
-from bitfount.federated.logging import _get_federated_logger
+    from bitfount.federated.transport.base_transport import Handler, _BaseMailbox
 
 logger = _get_federated_logger(__name__)
 
