@@ -47,7 +47,6 @@ from bitfount.config import get_gpu_metadata
 from bitfount.federated.encryption import _AESEncryption, _RSAEncryption
 from bitfount.federated.exceptions import PodConnectFailedError
 from bitfount.federated.logging import _get_federated_logger
-from bitfount.federated.transport.config import MessageServiceConfig
 from bitfount.federated.transport.exceptions import BitfountMessageServiceError
 from bitfount.federated.transport.protos.messages_pb2 import (
     BitfountMessage as GrpcBitfountMessage,
@@ -75,6 +74,7 @@ from bitfount.utils import _get_mb_from_bytes
 from bitfount.utils.concurrency_utils import await_threading_event
 
 if TYPE_CHECKING:
+    from bitfount.federated.transport.config import MessageServiceConfig
     from bitfount.hub.authentication_flow import BitfountSession
 
 logger = _get_federated_logger(__name__)
@@ -559,14 +559,7 @@ class _MessageService:
     @property
     def metadata(self) -> List[Tuple[str, str]]:
         """Returns metadata required for sending a BitfountMessage."""
-        if api_keys := self._session.api_keys:
-            return [
-                # Only the first portion of the API key is used
-                ("x-api-key-id", api_keys.access_key_id.split(":")[0]),
-                ("x-api-key", api_keys.access_key.split(":")[0]),
-            ]
-
-        return [("token", cast(str, self._session.access_token))]
+        return self._session.message_service_metadata
 
     async def connect_pod(self, pod_name: str) -> str:
         """Make the message service aware of the Pod.
@@ -712,7 +705,13 @@ class _MessageService:
                     upload_url,
                     upload_fields,
                 ), download_url = await self._get_large_object_storage_details(
-                    self._session._client_id, task_size, None, pod_name
+                    # TODO: [BIT-2607] The mailbox ID isn't available
+                    #  currently in this case. This will need to be tidied up
+                    #  when we fix the large message case
+                    "NOT_YET_AVAILABLE",
+                    task_size,
+                    None,
+                    pod_name,
                 )
                 await _LargeObjectRequestHandler.upload_large_object(
                     upload_url,
