@@ -15,7 +15,6 @@ from typing import (
 )
 import warnings
 
-from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
 import msgpack
 
@@ -24,6 +23,7 @@ from bitfount.federated.encryption import _RSAEncryption
 from bitfount.federated.logging import _federate_logger, _get_federated_logger
 from bitfount.federated.transport.base_transport import (
     Handler,
+    SyncHandler,
     _BaseMailbox,
     _send_aes_encrypted_message,
 )
@@ -158,7 +158,7 @@ class _WorkerMailbox(_BaseMailbox):
             error_messages: Error messages to send
 
         Returns:
-            True if message sent successfully, else false
+            True if message sent successfully, else False
 
         """
         logger.info(f"Rejecting task from {self.modeller_name}")
@@ -464,17 +464,36 @@ class _WorkerMailbox(_BaseMailbox):
     def _setup_federated_logging(self) -> None:
         """Set up federated logging."""
         _federate_logger(self)
+        self.register_handler(
+            _BitfountMessageType.LOG_MESSAGE, self._get_log_message_handler()
+        )
+
+    def _get_log_message_handler(self) -> SyncHandler:
+        """Create the appropriate handler for LOG_MESSAGE messages."""
 
         def log_message_handler(message: _BitfountMessage) -> None:
+            """Locally logs the log message that has been received from the modeller."""
             log_message: _JSONDict = self._aes_decrypt(message)
+
             # We prepend the log message to show it's come from the Modeller
-            log_message["msg"] = f"Modeller: {log_message['msg']}"
+            log_message["msg"] = f"<FROM MODELLER>: {log_message['msg']}"
+
+            # Modify processName and threadName to indicate these are non-local
+            try:
+                log_message["processName"] = f"<{log_message['processName']}>"
+            except KeyError:
+                pass
+            try:
+                log_message["threadName"] = f"<{log_message['threadName']}>"
+            except KeyError:
+                pass
+
             # We remove the `federated` key to avoid recursively sending a federated
             # log message on both the Modeller and Worker sides
             log_message.pop("federated")
             logger.handle(logging.makeLogRecord(log_message))
 
-        self.register_handler(_BitfountMessageType.LOG_MESSAGE, log_message_handler)
+        return log_message_handler
 
 
 async def _send_training_metrics(
@@ -525,10 +544,7 @@ async def _send_public_key(
         worker_mailbox: The worker mailbox to use to send the parameter update.
     """
     # Serialize the public key first
-    serialized_key = public_key.public_bytes(
-        encoding=serialization.Encoding.PEM,
-        format=serialization.PublicFormat.SubjectPublicKeyInfo,
-    )
+    serialized_key = _RSAEncryption.serialize_public_key(public_key)
     await worker_mailbox._send_aes_encrypted_message(
         serialized_key, _BitfountMessageType.KEY_EXCHANGE
     )
