@@ -35,11 +35,11 @@ _AGGREGATORS: Mapping[
 
 
 # This is a read-only dictionary mapping the name of an algorithm to the class itself
-_ALGORITHMS: Mapping[str, Type[algorithms._BaseAlgorithmFactory]] = algorithms.registry
+_ALGORITHMS: Mapping[str, Type[algorithms.BaseAlgorithmFactory]] = algorithms.registry
 
 
 # This is a read-only dictionary mapping the name of a protocol to the class itself
-_PROTOCOLS: Mapping[str, Type[protocols._BaseProtocolFactory]] = protocols.registry
+_PROTOCOLS: Mapping[str, Type[protocols.BaseProtocolFactory]] = protocols.registry
 
 
 _MODEL_STRUCTURES: Dict[
