@@ -1,6 +1,5 @@
 """Types for hub-related code."""
-from dataclasses import dataclass
-from typing import Dict, Final, Literal, TypedDict, Union
+from typing import Dict, Final, Literal, Optional, TypedDict, Union
 
 from typing_extensions import NotRequired
 
@@ -36,14 +35,6 @@ _DEV_IDP_URL: Final[str] = (
 )
 
 
-@dataclass
-class APIKeys:
-    """API keys for BitfountSession."""
-
-    access_key_id: str
-    access_key: str
-
-
 # General JSONs
 class _HubSuccessResponseJSON(TypedDict):
     """Generic hub success response JSON."""
@@ -158,6 +149,29 @@ class _MonitorPostJSON(TypedDict):
     resourceUsage: NotRequired[Dict[str, float]]
 
 
+class _RegisterUserPublicKeyPOSTJSON(TypedDict):
+    """Form of POST JSON for registering user public key.
+
+    API: POST /api/[username]/keys
+
+    The public key should be in OpenSSH format.
+    """
+
+    publicKey: str  # should be in ssh format
+
+
+class _UserIdentityVerifiersResponseJSON(TypedDict):
+    """Response JSON from GET /api/identity-verifiers.
+
+    Keys will be returned in PEM format.
+    """
+
+    username: str
+    samlUserID: str
+    oidcUserID: str
+    publicKey: NotRequired[Optional[str]]  # should be in PEM format, may not be present
+
+
 # Access Manager-related JSON
 class _AccessManagerKeyResponseJSON(TypedDict):
     """Response JSON from GET /api/access-manager-key."""
@@ -195,13 +209,17 @@ class _OIDCAccessCheckPostJSON(TypedDict):
 
 
 class _SignatureBasedAccessCheckPostJSON(TypedDict):
-    """Required keys for Signatured based JSON POST /api/access."""
+    """Required keys for Signatured based JSON POST /api/access.
+
+    NOTE: unsignedTask and taskSignature are byte-strings but will need to be b64
+          encoded to allow them to be sent as JSON.
+    """
 
     podIdentifier: str
     modellerName: str
-    modellerRequest: SerializedProtocol
-    unsignedTask: bytes
-    signedTask: bytes
+    modellerProtocolRequest: SerializedProtocol
+    unsignedTask: str  # b64 encoded byte-string
+    taskSignature: str  # b64 encoded byte-string
     identityProvider: Literal["SIGNATURE"]
 
 
