@@ -44,7 +44,6 @@ from bitfount.data.schema import BitfountSchema, TableSchema
 from bitfount.types import (
     T_FIELDS_DICT,
     T_NESTED_FIELDS,
-    T_OPTIONAL_NESTED,
     DistributedModelProtocol,
     _BaseSerializableObjectMixIn,
     _StrAnyDict,
@@ -66,12 +65,6 @@ logger = logging.getLogger(__name__)
 # Includes both abstract and non-abstract classes.
 MAIN_MODEL_REGISTRY: _StrAnyDict = {}
 
-# MYPY ISSUES
-# Due to the use of desert.field to create non-standard fields in these dataclasses,
-# mypy complains about incompatible type assignment. This is a known bug in desert
-# (https://github.com/python-desert/desert/issues/101) and so we can ignore it
-# until the issue is resolved.
-
 
 @dataclass
 class LoggerConfig:
@@ -97,11 +90,11 @@ class LoggerConfig:
             deserialize=lambda path: path if path is None else Path(path).expanduser()
         ),
         default=BITFOUNT_LOGS_DIR,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     #: same as argument
     params: Optional[_StrAnyDict] = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )  # type: ignore[assignment] # Reason: default_factory causes type issues
 
 
 class ModelContext(Enum):
@@ -323,7 +316,13 @@ class _BaseModel(
         """
         raise NotImplementedError
 
-    def predict(self, data: BaseSource, **kwargs: Any) -> np.ndarray:
+    @abstractmethod
+    def predict(
+        self,
+        data: Optional[BaseSource] = None,
+        pod_identifiers: Optional[List[str]] = None,
+        **kwargs: Any,
+    ) -> Union[List[np.ndarray], Dict[str, List[np.ndarray]]]:
         """This method runs inference on the test data, returns predictions.
 
         Args:
@@ -853,7 +852,6 @@ class NeuralNetworkMixIn(_BaseSerializableObjectMixIn):
         "scheduler": scheduler_registry,
         "early_stopping": early_stopping_registry,
     }
-    optional_nested: ClassVar[T_OPTIONAL_NESTED] = ["scheduler", "early_stopping"]
 
     def __init__(
         self,
