@@ -6,7 +6,6 @@ Models defined here cannot be trained in a federated manner.
 
 :::
 """
-
 from __future__ import annotations
 
 from dataclasses import dataclass
@@ -16,9 +15,9 @@ import os
 
 # TODO: [BIT-987] Review use of pickle
 import pickle  # nosec
-from typing import Any, List, Literal, Mapping, Optional, Tuple, Type, Union, cast
+from typing import Any, List, Literal, Mapping, Optional, Tuple, Union, cast
 
-from marshmallow import fields, post_load
+from marshmallow import fields
 import numpy as np
 import pandas as pd
 from pandas import DataFrame
@@ -27,15 +26,16 @@ from sklearn.linear_model import LogisticRegression as sklearnLogisticRegression
 from sklearn.neighbors import KNeighborsClassifier
 import statsmodels.api as sm
 
-from bitfount.data.dataloader import _BitfountDataLoader
-from bitfount.data.datasource import DataSource
+from bitfount.data.dataloaders import BitfountDataLoader
+from bitfount.data.datasources.base_source import BaseSource
 from bitfount.metrics import Metric
 from bitfount.models.base_models import ClassifierMixIn, RegressorMixIn, _BaseModel
-from bitfount.types import _DataFrameType, _JSONDict
+from bitfount.utils import delegates
 
 logger = logging.getLogger(__name__)
 
 
+@delegates()
 class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
     """Wrapper around `sklearn.linear_model.LogisticRegression` model.
 
@@ -58,6 +58,15 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         verbose: Verbosity level.
     """
 
+    fields_dict = {
+        "inverse_regularisation": fields.Float(allow_none=True),
+        "max_steps": fields.Integer(allow_none=True),
+        "model_type": fields.String(allow_none=True),
+        "penalty": fields.String(allow_none=True),
+        "early_stopping_tolerance": fields.Float(allow_none=True),
+        "verbose": fields.Integer(allow_none=True),
+    }
+
     def __init__(
         self,
         inverse_regularisation: Optional[float] = None,
@@ -69,7 +78,6 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         **kwargs: Any,
     ):
         super().__init__(**kwargs)
-
         self.inverse_regularisation = (
             0.0001 if inverse_regularisation is None else inverse_regularisation
         )
@@ -81,6 +89,11 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         )
         self.verbose = 0 if verbose is None else verbose
 
+    def predict(self, *args: Any, **kwargs: Any) -> List[np.ndarray]:
+        """Returns model predictions. Not implemented yet."""
+        # TODO: [BIT-2406] Implement this method
+        raise NotImplementedError
+
     def serialize(self, filename: Union[str, os.PathLike]) -> None:
         """Serialize model to file with provided `filename`.
 
@@ -93,28 +106,28 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
     def deserialize(self, filename: Union[str, os.PathLike]) -> None:
         """Deserialize model.
 
-        Args:
-            filename: Path to file containing serialized model.
-
         :::danger
 
         This should not be used on a model file that has been received across a
         trust boundary due to underlying use of `pickle`.
 
         :::
+
+        Args:
+            filename: Path to file containing serialized model.
         """
         with open(filename, "rb") as f:
             # TODO: [BIT-987] Review use of pickle.load()
             self._model = pickle.load(f)  # nosec
 
     def evaluate(
-        self, test_dl: Optional[_BitfountDataLoader] = None, *args: Any, **kwargs: Any
+        self, test_dl: Optional[BitfountDataLoader] = None, *args: Any, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """Perform inference on test set and return predictions and targets.
 
         Args:
             test_dl: Optional `BitfountDataLoader` object containing test data. If this
-                is not provided, the test set from the `DataSource` used to train the
+                is not provided, the test set from the `BaseSource` used to train the
                 model is used if present.
 
         Returns:
@@ -126,13 +139,13 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         logger.info("Evaluating logistic regression classifier.")
 
         if test_dl is None:
-            if isinstance(self.test_dl, _BitfountDataLoader):
+            if isinstance(self.test_dl, BitfountDataLoader):
                 test_dl = self.test_dl
             else:
                 raise ValueError("There is no test data to evaluate the model on.")
         test_df = test_dl.get_x_dataframe()
         test_preds: np.ndarray = cast(
-            Type[sklearnLogisticRegression], self._model
+            sklearnLogisticRegression, self._model
         ).predict_proba(test_df)
         test_target = test_dl.get_y_dataframe().to_numpy()
 
@@ -140,20 +153,20 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
 
     def fit(
         self,
-        data: Optional[DataSource] = None,
+        data: Optional[BaseSource] = None,
         *args: Any,
         **kwargs: Any,
     ) -> None:
         """Trains a model using the training set provided by `data`.
 
-        Args:
-            data: `DataSource` object containing training data.
-
         :::info
 
         The validation set in `data` is not used when training this model.
 
         :::
+
+        Args:
+            data: `BaseSource` object containing training data.
         """
         logger.info("Fitting logistic regression classifier.")
         if data:
@@ -175,40 +188,12 @@ class LogisticRegressionClassifier(ClassifierMixIn, _BaseModel):
         )
         model.fit(
             self.train_dl.get_x_dataframe(),
-            self.train_dl.get_y_dataframe()[self._databunch.target].to_numpy(),
+            self.train_dl.get_y_dataframe()[self.databunch.target].to_numpy(),
         )
         self._model = model
 
-    @property
-    def training_needed(self) -> bool:
-        """Dictates whether the model needs training.
-
-        Returns:
-            bool: True if `self.max_steps` is greater than 0.
-        """
-        return self.max_steps > 0
-
-    class _Schema(_BaseModel._Schema, ClassifierMixIn._Schema):
-        inverse_regularisation = fields.Float(allow_none=True)
-        max_steps = fields.Integer(allow_none=True)
-        model_type = fields.String(allow_none=True)
-        penalty = fields.String(allow_none=True)
-        early_stopping_tolerance = fields.Float(allow_none=True)
-        verbose = fields.Integer(allow_none=True)
-
-        @post_load
-        def recreate_model(
-            self, data: _JSONDict, **kwargs: Any
-        ) -> LogisticRegressionClassifier:
-            """Recreate model from Schema."""
-            return LogisticRegressionClassifier(**data)
-
-    @classmethod
-    def get_schema(cls) -> Type[LogisticRegressionClassifier._Schema]:
-        """Get the model schema."""
-        return cls._Schema
-
 
+@delegates()
 class RegBoostRegressor(RegressorMixIn, _BaseModel):
     """Gradient Boosted Linear Regression Model.
 
@@ -238,6 +223,14 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
         stepwise_regression_threshold: Threshold for stepwise regression.
     """
 
+    fields_dict = {
+        "learning_rate": fields.Float(allow_none=True),
+        "max_depth": fields.Integer(allow_none=True),
+        "min_data_points_per_node": fields.Integer(allow_none=True),
+        "stepwise_regression": fields.String(allow_none=True),
+        "stepwise_regression_threshold": fields.Float(allow_None=True),
+    }
+
     def __init__(
         self,
         learning_rate: float = 0.1,
@@ -248,13 +241,17 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
         **kwargs: Any,
     ):
         super().__init__(**kwargs)
-
         self.learning_rate = learning_rate
         self.max_depth = max_depth
         self.min_data_points_per_node = min_data_points_per_node
         self.stepwise_regression = stepwise_regression
         self.stepwise_regression_threshold = stepwise_regression_threshold
 
+    def predict(self, *args: Any, **kwargs: Any) -> List[np.ndarray]:
+        """Returns model predictions. Not implemented yet."""
+        # TODO: [BIT-2406] Implement this method
+        raise NotImplementedError
+
     class _ModelTreeSide(Enum):
         """Represents the side which a node represents in ModelTree."""
 
@@ -277,7 +274,9 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
 
         def __post_init__(self) -> None:
             self.model = sklearnLinearRegression()
-            self.classifier = None  # only used at inference time
+            self.classifier: Optional[
+                KNeighborsClassifier
+            ] = None  # only used at inference time
             self.features: List[str] = []
             self.negatives: Optional[RegBoostRegressor._ModelTree] = None
             self.positives: Optional[RegBoostRegressor._ModelTree] = None
@@ -303,9 +302,9 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
             """Call self.model fit method."""
             return self.model.fit(*args, **kwargs)
 
-        def predict(self, *args: Any, **kwargs: Any) -> np.ndarray:
+        def predict(self, *args: Any, **kwargs: Any) -> List[np.ndarray]:
             """Call self.model predict method."""
-            return cast(np.ndarray, self.model.predict(*args, **kwargs))
+            return [np.asarray(i) for i in self.model.predict(*args, **kwargs)]
 
         @property
         def is_leaf(self) -> bool:
@@ -378,7 +377,7 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
 
         # If we haven't reached the maximum depth, create positive and negative children
         if model.depth < self.max_depth:
-            preds = model.predict(X)
+            preds = np.asarray(model.predict(X))
 
             for side, indices in zip(
                 RegBoostRegressor._ModelTreeSide,
@@ -488,12 +487,12 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
 
     def fit(
         self,
-        data: Optional[DataSource] = None,
+        data: Optional[BaseSource] = None,
         metrics: Optional[Mapping[str, Metric]] = None,
         *args: Any,
         **kwargs: Any,
     ) -> None:
-        """Trains a model using the training set provided by the DataSource object."""
+        """Trains a model using the training set provided by the BaseSource object."""
         logger.info("Fitting RegBoost regressor.")
         self.initialise_model(data)
         self._set_dataloaders()
@@ -502,13 +501,13 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
         self._model: RegBoostRegressor._ModelTree = model
 
     def evaluate(
-        self, test_dl: Optional[_BitfountDataLoader] = None, *args: Any, **kwargs: Any
+        self, test_dl: Optional[BitfountDataLoader] = None, *args: Any, **kwargs: Any
     ) -> Tuple[np.ndarray, np.ndarray]:
         """Perform inference on test set and return predictions and targets.
 
         Args:
             test_dl: Optional `BitfountDataLoader` object containing test data. If this
-                is not provided, the test set from the `DataSource` used to train the
+                is not provided, the test set from the `BaseSource` used to train the
                 model is used if present.
 
         Returns:
@@ -521,13 +520,20 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
 
         k = cast(int, kwargs.get("k"))
         if test_dl is None:
-            if isinstance(self.test_dl, _BitfountDataLoader):
+            if isinstance(self.test_dl, BitfountDataLoader):
                 test_dl = self.test_dl
             else:
                 raise ValueError("There is no test data to evaluate the model on.")
-        test_df: _DataFrameType = test_dl.get_x_dataframe()
+
+        test_df = test_dl.get_x_dataframe()
+        if isinstance(test_df, tuple):
+            raise ValueError(
+                "Multiple dataframes retrieved unexpectedly; "
+                "this model does not support combination tabular and image data."
+            )
+
         test_target = (
-            test_dl.get_y_dataframe()[self._databunch.target]
+            test_dl.get_y_dataframe()[self.databunch.target]
             .to_numpy()
             .astype(np.float32)
         )
@@ -544,7 +550,9 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
             self._fit_model_tree_classifier(model, k)
         test_df = test_df[model.features]
         tree_classes = self._eval_model_tree_classifier(model, test_df)
-        preds = np.reshape(model.predict(test_df), (len(test_df), 1)).tolist()
+        preds = np.reshape(
+            np.asarray(model.predict(test_df)), (len(test_df), 1)
+        ).tolist()
 
         # Split data into positives and negatives based on classifier results
         for indices, model_ in zip(
@@ -592,7 +600,14 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
         """Fit ModelTree classifier with training data."""
         if model.negatives is not None and model.positives is not None:
             model.classifier = KNeighborsClassifier(n_neighbors=k)
-            X: _DataFrameType = self.train_dl.get_x_dataframe()
+
+            X = self.train_dl.get_x_dataframe()
+            if isinstance(X, tuple):
+                raise ValueError(
+                    "Multiple dataframes retrieved unexpectedly; "
+                    "this model does not support combination tabular and image data."
+                )
+
             X = X[model.features].loc[
                 np.concatenate(
                     [model.positives.data_indices, model.negatives.data_indices]
@@ -633,50 +648,17 @@ class RegBoostRegressor(RegressorMixIn, _BaseModel):
     def deserialize(self, filename: Union[str, os.PathLike]) -> None:
         """Deserialize model.
 
-        Args:
-            filename: Path to file containing serialized model.
-
         :::danger
 
         This should not be used on a model file that has been received across a
         trust boundary due to underlying use of `pickle`.
 
         :::
+
+        Args:
+            filename: Path to file containing serialized model.
         """
         self._set_dataloaders()
         with open(filename, "rb") as f:
             # TODO: [BIT-987] Review use of pickle.load()
             self._model = pickle.load(f)  # nosec
-
-    @property
-    def training_needed(self) -> bool:
-        """Dictates whether the model needs training.
-
-        Returns:
-            bool: True if model is untrained, False otherwise.
-        """
-        return (
-            not isinstance(self._model, RegBoostRegressor._ModelTree)
-            or self._model.is_leaf
-        )
-
-    class _Schema(_BaseModel._Schema):
-        learning_rate = fields.Float(allow_none=True)
-        max_depth = fields.Integer(allow_none=True)
-        min_data_points_per_node = fields.Integer(allow_none=True)
-        stepwise_regression = fields.String(allow_none=True)
-        stepwise_regression_threshold = fields.Float(allow_None=True)
-
-        @post_load
-        def recreate_model(self, data: _JSONDict, **kwargs: Any) -> RegBoostRegressor:
-            """Recreate model from Schema."""
-            return RegBoostRegressor(**data)
-
-    @classmethod
-    def get_schema(cls) -> Type[RegBoostRegressor._Schema]:
-        """Get the model schema.
-
-        Returns:
-            Type[RegBoostRegressor._Schema]: Schema for model.
-        """
-        return cls._Schema
