@@ -2,12 +2,13 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
+from datetime import datetime
 import logging
 import os
 from pathlib import Path
 import re
 import typing
-from typing import Dict, List, Mapping, MutableMapping, Optional, Union
+from typing import Callable, Dict, List, Mapping, MutableMapping, Optional, Tuple, Union
 
 import desert
 from marshmallow import ValidationError, fields, validate
@@ -37,7 +38,7 @@ from bitfount.federated.transport.config import (
     _STAGING_MESSAGE_SERVICE_URL,
     MessageServiceConfig,
 )
-from bitfount.hub.authentication_flow import _DEFAULT_USERNAME
+from bitfount.hub.authentication_handlers import _DEFAULT_USERNAME
 from bitfount.hub.types import (
     _DEV_AM_URL,
     _DEV_HUB_URL,
@@ -48,7 +49,6 @@ from bitfount.hub.types import (
     _STAGING_IDP_URL,
     PRODUCTION_AM_URL,
     PRODUCTION_HUB_URL,
-    APIKeys,
 )
 from bitfount.models.base_models import (
     EarlyStopping,
@@ -66,22 +66,33 @@ logger = logging.getLogger(__name__)
 POD_NAME_REGEX = re.compile(r"[a-z\d]+((-|_)[a-z\d]+)*")
 USERNAME_REGEX = re.compile(r"[a-z\d](?:[a-z\d]|(-|_)(?=[a-z\d])){0,38}")
 
-# MYPY ISSUES
-# Due to the use of desert.field to create non-standard fields in these dataclasses,
-# mypy complains about incompatible type assignment. This is a known bug in desert
-# (https://github.com/python-desert/desert/issues/101) and so we can ignore it
-# until the issue is resolved.
 
-
-def _deserialize_path(path: Optional[str]) -> Optional[Path]:
+def _deserialize_path(
+    path: Optional[str], context: Dict[str, typing.Any]
+) -> Optional[Path]:
     """Converts a str into a Path.
 
     If the input is None, the output is None.
+
+    If the path to the config file is supplied in the `context` dict (in the
+    "config_path" key) then any relative paths will be resolved relative to the
+    directory containing the config file.
     """
     if path is None:
-        return path
-    else:
-        return Path(path).expanduser()
+        return None
+
+    ppath = Path(path)
+
+    # If relative path, use relative to config file if present
+    if not ppath.is_absolute() and "config_path" in context:
+        config_dir = Path(context["config_path"]).parent
+        orig_ppath = ppath
+        ppath = config_dir.joinpath(ppath).resolve()
+        logger.debug(
+            f"Making relative paths relative to {config_dir}: {orig_ppath} -> {ppath}"
+        )
+
+    return ppath.expanduser()
 
 
 def _deserialize_model_ref(ref: str) -> Union[Path, str]:
@@ -102,18 +113,31 @@ def _deserialize_model_ref(ref: str) -> Union[Path, str]:
 class AccessManagerConfig:
     """Configuration for the access manager."""
 
-    url: str = desert.field(  # type: ignore[assignment] # Reason: see top of file
-        fields.URL(), default=PRODUCTION_AM_URL
-    )
+    url: str = desert.field(fields.URL(), default=PRODUCTION_AM_URL)
 
 
 @dataclass
 class HubConfig:
     """Configuration for the hub."""
 
-    url: str = desert.field(  # type: ignore[assignment] # Reason: see top of file
-        fields.URL(), default=PRODUCTION_HUB_URL
-    )
+    url: str = desert.field(fields.URL(), default=PRODUCTION_HUB_URL)
+
+
+@dataclass
+class APIKeys:
+    """API keys for BitfountSession."""
+
+    access_key_id: str = desert.field(fields.String())
+    access_key: str = desert.field(fields.String())
+
+
+@dataclass
+class JWT:
+    """Externally managed JWT for BitfountSession."""
+
+    jwt: str = desert.field(fields.String())
+    expires: datetime = desert.field(fields.DateTime())
+    get_token: Callable[[], Tuple[str, datetime]] = desert.field(fields.Function())
 
 
 # MODELLER SCHEMAS
@@ -134,13 +158,13 @@ class ModellerUserConfig:
     username: str = desert.field(
         fields.String(validate=validate.Regexp(USERNAME_REGEX)),
         default=_DEFAULT_USERNAME,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
 
     identity_verification_method: str = desert.field(
         fields.String(validate=OneOf(IDENTITY_VERIFICATION_METHODS)),
         default=DEFAULT_IDENTITY_VERIFICATION_METHOD,
-    )  # type: ignore[assignment] # Reason: see top of file
-    private_key_file: Optional[Path] = desert.field(  # type: ignore[assignment] # Reason: see top of file  # noqa: B950
+    )
+    private_key_file: Optional[Path] = desert.field(
         fields.Function(deserialize=_deserialize_path), default=None
     )
 
@@ -161,20 +185,29 @@ class ModellerConfig:
     """Full configuration for the modeller."""
 
     pods: PodsConfig
-    data_structure: DataStructureConfig
     task: TaskConfig
+    secrets: Optional[Union[APIKeys, JWT]] = desert.field(
+        M_Union(
+            [
+                fields.Nested(desert.schema_class(APIKeys)),
+                fields.Nested(desert.schema_class(JWT)),
+            ]
+        ),
+        default=None,
+    )
 
-    modeller: ModellerUserConfig = desert.field(  # type: ignore[assignment] # Reason: see top of file  # noqa: B950
+    modeller: ModellerUserConfig = desert.field(
         fields.Nested(desert.schema_class(ModellerUserConfig)),
         default_factory=ModellerUserConfig,
     )
-    hub: HubConfig = desert.field(  # type: ignore[assignment] # Reason: see top of file
+    hub: HubConfig = desert.field(
         fields.Nested(desert.schema_class(HubConfig)), default_factory=HubConfig
     )
     message_service: MessageServiceConfig = desert.field(
         fields.Nested(desert.schema_class(MessageServiceConfig)),
         default_factory=MessageServiceConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
+    project_id: Optional[str] = None
 
     def __post_init__(self) -> None:
         environment = _get_environment()
@@ -195,7 +228,7 @@ class ModelStructureConfig:
     name: str
     arguments: _JSONDict = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
 
 
 @dataclass
@@ -204,7 +237,7 @@ class BitfountModelReferenceConfig:
 
     model_ref: Union[Path, str] = desert.field(
         fields.Function(deserialize=_deserialize_model_ref)
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     model_version: Optional[int] = None
     username: Optional[str] = None
 
@@ -223,7 +256,7 @@ class ModelConfig:
     # Other
     hyperparameters: _JSONDict = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     logger_config: Optional[LoggerConfig] = None
     dp_config: Optional[DPModellerConfig] = None
 
@@ -275,14 +308,7 @@ class ProtocolConfig:
     name: str
     arguments: _JSONDict = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
-
-    def __post_init__(self) -> None:
-        # TODO: [BIT-1300] Remove this once custom protocols are supported
-        # This is added for convenience so the user doesn't have to be verbose with the
-        # protocol name when there are only bitfount protocols available
-        if not self.name.startswith("bitfount."):
-            self.name = f"bitfount.{self.name}"
+    )
 
 
 @dataclass
@@ -307,17 +333,11 @@ class AlgorithmConfig:
     name: str
     arguments: _JSONDict = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     pretrained_file: Optional[Path] = desert.field(
         fields.Function(deserialize=_deserialize_path), default=None
-    )  # type: ignore[assignment] # Reason: see top of file
-
-    def __post_init__(self) -> None:
-        # TODO: [BIT-1300] Remove this once custom algorithms are supported
-        # This is added for convenience so the user doesn't have to be verbose with the
-        # algorithm name when there are only bitfount algorithms available
-        if not self.name.startswith("bitfount."):
-            self.name = f"bitfount.{self.name}"
+    )
+    model: Optional[ModelConfig] = None
 
 
 @dataclass
@@ -325,12 +345,12 @@ class TaskConfig:
     """Configuration for the task."""
 
     protocol: ProtocolConfig
-    algorithm: AlgorithmConfig
-    model: ModelConfig
+    algorithm: Union[AlgorithmConfig, List[AlgorithmConfig]]
+    data_structure: Optional[DataStructureConfig] = None
     aggregator: Optional[AggregatorConfig] = None
     transformation_file: Optional[Path] = desert.field(
         fields.Function(deserialize=_deserialize_path), default=None
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
 
 
 # POD SCHEMAS
@@ -344,32 +364,40 @@ class PodConfig:
 
     datasource: str
     data_config: PodDataConfig
-    name: str = desert.field(
-        fields.String(validate=validate.Regexp(POD_NAME_REGEX))
-    )  # type: ignore[assignment] # Reason: see top of file
-    api_key: Optional[APIKeys] = None
+    name: str = desert.field(fields.String(validate=validate.Regexp(POD_NAME_REGEX)))
+    secrets: Optional[Union[APIKeys, JWT]] = desert.field(
+        M_Union(
+            [
+                fields.Nested(desert.schema_class(APIKeys)),
+                fields.Nested(desert.schema_class(JWT)),
+            ]
+        ),
+        default=None,
+    )
     pod_details_config: Optional[PodDetailsConfig] = None
     schema: Optional[Path] = desert.field(
         fields.Function(deserialize=_deserialize_path), default=None
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     access_manager: AccessManagerConfig = desert.field(
         fields.Nested(desert.schema_class(AccessManagerConfig)),
         default_factory=AccessManagerConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
-    hub: HubConfig = desert.field(  # type: ignore[assignment] # Reason: see top of file
+    )
+    hub: HubConfig = desert.field(
         fields.Nested(desert.schema_class(HubConfig)), default_factory=HubConfig
     )
     message_service: MessageServiceConfig = desert.field(
         fields.Nested(desert.schema_class(MessageServiceConfig)),
         default_factory=MessageServiceConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     differential_privacy: Optional[DPPodConfig] = None
     approved_pods: Optional[List[str]] = None
     username: str = desert.field(
         fields.String(validate=validate.Regexp(USERNAME_REGEX)),
         default=_DEFAULT_USERNAME,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     update_schema: bool = False
+    pod_db: bool = False
+    show_datapoints_with_results_in_db: bool = True
 
     def __post_init__(self) -> None:
         environment = _get_environment()
@@ -385,7 +413,7 @@ class PodConfig:
             self.message_service.tls = _DEV_MESSAGE_SERVICE_TLS
 
         # Use API Keys for authentication if provided
-        if self.api_key:
+        if isinstance(self.secrets, APIKeys):
             if self.username == _DEFAULT_USERNAME:
                 raise ValueError("Must specify a username when using API Keys.")
 
@@ -398,8 +426,8 @@ class PodConfig:
                     "Existing environment variable API keys detected. Overriding with "
                     "those provided in the pod config."
                 )
-            os.environ["BITFOUNT_API_KEY_ID"] = self.api_key.access_key_id
-            os.environ["BITFOUNT_API_KEY"] = self.api_key.access_key
+            os.environ["BITFOUNT_API_KEY_ID"] = self.secrets.access_key_id
+            os.environ["BITFOUNT_API_KEY"] = self.secrets.access_key
 
     @property
     def pod_id(self) -> str:
@@ -411,9 +439,7 @@ class PodConfig:
 class PathConfig:
     """Configuration for the path."""
 
-    path: Path = desert.field(
-        fields.Function(deserialize=_deserialize_path)
-    )  # type: ignore[assignment] # Reason: see top of file
+    path: Path = desert.field(fields.Function(deserialize=_deserialize_path))
 
 
 @dataclass
@@ -452,10 +478,8 @@ class DataSplitConfig:
     data_splitter: str = desert.field(
         fields.String(validate=OneOf(["percentage", "predefined"])),
         default="percentage",
-    )  # type: ignore[assignment] # Reason: see top of file
-    args: _JSONDict = desert.field(
-        fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
+    args: _JSONDict = desert.field(fields.Dict(keys=fields.Str), default_factory=dict)
 
 
 @dataclass
@@ -481,11 +505,6 @@ class PodDataConfig:
             values. Defaults to False.
     """
 
-    # There is a bug in desert (https://github.com/python-desert/desert/issues/122)
-    # which causes explicitly Optional Union fields to error out. Our mypy config
-    # requires explicit Optional for good practice so we currently have to do more
-    # involved desert.field() assignments until the issue is fixed.
-
     force_stypes: Optional[
         Mapping[str, MutableMapping[_SemanticTypeValue, List[str]]]
     ] = desert.field(
@@ -497,12 +516,12 @@ class PodDataConfig:
             ),
         ),
         default=None,
-    )  # type: ignore[assignment] # Reason: see comment and top of file
+    )
 
     ignore_cols: Optional[Mapping[str, List[str]]] = desert.field(
         fields.Dict(keys=fields.String(), values=fields.List(fields.String())),
         default=None,
-    )  # type: ignore[assignment] # Reason: see comment and top of file
+    )
 
     modifiers: Optional[Dict[str, DataPathModifiers]] = desert.field(
         fields.Dict(
@@ -515,14 +534,14 @@ class PodDataConfig:
             default=None,
         ),
         default=None,
-    )  # type: ignore[assignment] # Reason: see comment and top of file
+    )
     datasource_args: _JSONDict = desert.field(
         fields.Dict(keys=fields.Str), default_factory=dict
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     data_split: DataSplitConfig = desert.field(
         fields.Nested(desert.schema_class(DataSplitConfig)),
         default_factory=DataSplitConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     auto_tidy: bool = False
 
 
@@ -545,20 +564,19 @@ class DataStructureSelectConfig:
 
     include: List[str] = desert.field(
         fields.List(fields.String()), default_factory=list
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     exclude: List[str] = desert.field(
         fields.List(fields.String()), default_factory=list
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
 
 
 @dataclass
 class DataStructureAssignConfig:
     """Configuration for the datastructure assign argument."""
 
-    target: Union[str, List[str]] = desert.field(
-        M_Union([fields.String(), fields.List(fields.String())])
-    )  # type: ignore[assignment] # Reason: see top of file
-    # When we add more DataStructures, we should make the target an optional argument
+    target: Optional[Union[str, List[str]]] = desert.field(
+        M_Union([fields.String(), fields.List(fields.String())]), default=None
+    )
     image_cols: Optional[List[str]] = None
     loss_weights_col: Optional[str] = None
     multihead_col: Optional[str] = None
@@ -571,6 +589,7 @@ class DataStructureTransformConfig:
 
     dataset: Optional[List[Dict[str, _JSONDict]]] = None
     batch: Optional[List[Dict[str, _JSONDict]]] = None
+    auto_convert_grayscale_images: bool = True
 
 
 @dataclass
@@ -586,7 +605,7 @@ class DataStructureTableConfig:
             default=None,
         ),
         default=None,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     query: Optional[Union[str, Dict[str, str]]] = desert.field(
         M_Union(
             [
@@ -596,7 +615,7 @@ class DataStructureTableConfig:
             default=None,
         ),
         default=None,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     schema_types_override: Optional[
         Union[SchemaOverrideMapping, Mapping[str, SchemaOverrideMapping]]
     ] = desert.field(
@@ -608,7 +627,7 @@ class DataStructureTableConfig:
             default=None,
         ),
         default=None,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
 
 
 @dataclass
@@ -617,15 +636,15 @@ class DataStructureConfig:
 
     table_config: DataStructureTableConfig = desert.field(
         fields.Nested(desert.schema_class(DataStructureTableConfig)),
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     assign: DataStructureAssignConfig = desert.field(
         fields.Nested(desert.schema_class(DataStructureAssignConfig)),
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     select: DataStructureSelectConfig = desert.field(
         fields.Nested(desert.schema_class(DataStructureSelectConfig)),
         default_factory=DataStructureSelectConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
     transform: DataStructureTransformConfig = desert.field(
         fields.Nested(desert.schema_class(DataStructureTransformConfig)),
         default_factory=DataStructureTransformConfig,
-    )  # type: ignore[assignment] # Reason: see top of file
+    )
