@@ -26,8 +26,10 @@ def run(
     model_out: Path = DEFAULT_MODEL_OUT,
 ) -> None:
     """Runs a modeller from a config file."""
-    modeller, pod_identifiers = setup_modeller_from_config_file(path_to_config_yaml)
-    run_modeller(modeller, pod_identifiers, require_all_pods, model_out)
+    modeller, pod_identifiers, project_id = setup_modeller_from_config_file(
+        path_to_config_yaml
+    )
+    run_modeller(modeller, pod_identifiers, require_all_pods, model_out, project_id)
 
 
 def main() -> None:
