@@ -1,6 +1,6 @@
 """Test protocol.py with pytorch backend."""
 import copy
-from typing import Any, Dict, List, Optional, Union, cast
+from typing import List, Optional, Union, cast
 from unittest.mock import AsyncMock, Mock, create_autospec
 
 from _pytest.logging import LogCaptureFixture
@@ -8,9 +8,9 @@ import pytest
 from pytest import fixture
 from pytest_mock import MockerFixture
 
-from bitfount.backends.pytorch.federated.models import PyTorchBackendTensorShim
+from bitfount.backends.pytorch.federated.shim import PyTorchBackendTensorShim
 from bitfount.backends.pytorch.models.models import PyTorchTabularClassifier
-from bitfount.data.datasource import DataSource
+from bitfount.data.datasources.dataframe_source import DataFrameSource
 from bitfount.data.datastructure import DataStructure
 from bitfount.data.schema import BitfountSchema
 from bitfount.federated.aggregators.aggregator import (
@@ -24,14 +24,14 @@ from bitfount.federated.algorithms.model_algorithms.federated_training import (
     FederatedModelTraining,
     _BaseModelTrainingMixIn,
 )
-from bitfount.federated.protocols.fed_avg import (
+from bitfount.federated.protocols.model_protocols.federated_averaging import (
     FederatedAveraging,
     _FederatedAveragingCompatibleAlgoFactory,
 )
 from bitfount.federated.secure import SecureShare
 from bitfount.federated.transport.modeller_transport import _ModellerMailbox
 from bitfount.federated.transport.worker_transport import _WorkerMailbox
-from bitfount.types import DistributedModelProtocol, _SerializedWeights
+from bitfount.types import DistributedModelProtocol, _SerializedWeights, _StrAnyDict
 from tests.bitfount import TEST_SECURITY_FILES
 from tests.utils import PytestRequest
 from tests.utils.helper import (
@@ -44,10 +44,21 @@ from tests.utils.helper import (
 
 
 @fixture
-def datasource() -> DataSource:
+def datasource() -> DataFrameSource:
     """Returns datasource."""
     dataset = create_dataset()
-    return DataSource(data_ref=dataset)
+    return DataFrameSource(dataset)
+
+
+@fixture
+def datastructure_mult_pods() -> DataStructure:
+    """Fixture for datastructure."""
+    ds = create_datastructure()
+    ds.table = {
+        "bitfount/census-income": "census-income",
+        "bitfount/census-income-2": "census-income-2",
+    }
+    return ds
 
 
 @fixture
@@ -65,14 +76,12 @@ def schema() -> BitfountSchema:
 @fixture
 def aggregator(request: PytestRequest) -> Union[Aggregator, SecureAggregator]:
     """Returns mock aggregator."""
-    tensor_shim = PyTorchBackendTensorShim()
     # request.param denotes whether we should use secure aggregator
     if request.param:
         return SecureAggregator(
-            tensor_shim=tensor_shim,
-            secure_share=SecureShare(tensor_shim=tensor_shim),
+            secure_share=SecureShare(),
         )
-    return Aggregator(tensor_shim=tensor_shim)
+    return Aggregator()
 
 
 @fixture
@@ -95,14 +104,15 @@ def mock_aggregator() -> Mock:
 def mock_modeller_mailbox() -> Mock:
     """Returns mock mailbox."""
     mock_mailbox: Mock = create_autospec(_ModellerMailbox, instance=True)
+    mock_mailbox._task_id = "task_id"
     return mock_mailbox
 
 
 @fixture
-def model(datastructure: DataStructure) -> PyTorchTabularClassifier:
+def model(datastructure_mult_pods: DataStructure) -> PyTorchTabularClassifier:
     """Returns distributed model."""
     return PyTorchTabularClassifier(
-        datastructure=datastructure, schema=BitfountSchema(), epochs=1
+        datastructure=datastructure_mult_pods, schema=BitfountSchema(), epochs=1
     )
 
 
@@ -162,7 +172,7 @@ def mock_federated_algorithm(request: PytestRequest) -> Mock:
 def mock_modeller_run_method(mocker: MockerFixture) -> AsyncMock:
     """Mocks out the Modeller.run() method in protocol.py."""
     mock_run_method: AsyncMock = mocker.patch(
-        "bitfount.federated.protocols.base.Modeller.run"
+        "bitfount.federated.protocols.base._Modeller.run"
     )
     return mock_run_method
 
@@ -170,7 +180,7 @@ def mock_modeller_run_method(mocker: MockerFixture) -> AsyncMock:
 @fixture
 def pod_identifiers() -> List[str]:
     """A list of pod identifiers."""
-    return ["bitfount/adult", "bitfount/adult2"]
+    return ["bitfount/census-income", "bitfount/census-income-2"]
 
 
 @backend_test
@@ -259,7 +269,7 @@ class TestFederatedAveraging:
     )
     async def test_worker_model_training_for_appropriate_amount(
         self,
-        datasource: DataSource,
+        datasource: DataFrameSource,
         epochs_between_parameter_updates: Optional[int],
         federated_algorithm: FederatedModelTraining,
         mock_aggregator: Mock,
@@ -271,10 +281,10 @@ class TestFederatedAveraging:
         # Create mailbox and mock out relevant functions
         mock_worker_mailbox = create_autospec(_WorkerMailbox, instance=True)
         mock_get_model_updates = mocker.patch(
-            "bitfount.federated.protocols.fed_avg._get_model_parameters"
+            "bitfount.federated.protocols.model_protocols.federated_averaging._get_model_parameters"
         )
         mock_send_model_updates = mocker.patch(
-            "bitfount.federated.protocols.fed_avg._send_parameter_update"
+            "bitfount.federated.protocols.model_protocols.federated_averaging._send_parameter_update"
         )
         # Mock out so training complete never returns truthy value prematurely
         mock_worker_mailbox.get_training_iteration_complete_update.return_value = False
@@ -297,12 +307,12 @@ class TestFederatedAveraging:
             _mailbox: _WorkerMailbox,
         ) -> _SerializedWeights:
             """Mock out get_model_parameters."""
-            model_params: Dict[str, Any] = copy.deepcopy(
-                cast(_BaseModelTrainingMixIn, protocol.algorithm).get_param_states()
-            )
+            params = cast(
+                _BaseModelTrainingMixIn, protocol.algorithm
+            ).get_param_states()
+            model_params = copy.deepcopy(cast(_StrAnyDict, params))
             for name, param in model_params.items():
                 model_params[name] = PyTorchBackendTensorShim.to_list(param)
-
             return model_params
 
         mock_get_model_updates.side_effect = mock_get_model_updates_coroutine
@@ -322,9 +332,11 @@ class TestFederatedAveraging:
         # number of iterations is correct.
         assert isinstance(federated_algorithm.model, DistributedModelProtocol)
         if epochs_between_parameter_updates:
+            # The `current_epoch` attribute denotes the epoch number of the epoch
+            # about to be trained on the next iteration (starting from 0)
             assert (
                 federated_algorithm.model._pl_trainer.current_epoch  # type: ignore[attr-defined] # Reason: Model has _pl_trainer # noqa: B950
-                == epochs_between_parameter_updates - 1
+                == epochs_between_parameter_updates
             )
         else:
             assert (
@@ -341,13 +353,16 @@ class TestFederatedAveraging:
         pod_identifiers: List[str],
     ) -> None:
         """Tests protocol helper run method with algorithm."""
-        FederatedAveraging.run(
+        protocol = FederatedAveraging(algorithm=federated_algorithm_with_model)
+
+        protocol.run(
             pod_identifiers=pod_identifiers,
-            algorithm=federated_algorithm_with_model,
             private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
         )
 
-        mock_modeller_run_method.assert_called_once_with(pod_identifiers)
+        mock_modeller_run_method.assert_called_once_with(
+            pod_identifiers, require_all_pods=False
+        )
 
     @unit_test
     def test_helper_run_method_with_model(
@@ -358,13 +373,15 @@ class TestFederatedAveraging:
         pod_identifiers: List[str],
     ) -> None:
         """Tests helper run method with a model."""
-        FederatedAveraging.run(
+        protocol = FederatedAveraging(algorithm=FederatedModelTraining(model=model))
+        protocol.run(
             pod_identifiers=pod_identifiers,
-            model=model,
             private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
         )
 
-        mock_modeller_run_method.assert_called_once_with(pod_identifiers)
+        mock_modeller_run_method.assert_called_once_with(
+            pod_identifiers, require_all_pods=False
+        )
 
     @unit_test
     def test_helper_run_method_with_model_and_algorithm(
@@ -381,15 +398,17 @@ class TestFederatedAveraging:
         regarding the extra model argument.
         """
         mock_model = Mock()
-
-        FederatedAveraging.run(
+        protocol = FederatedAveraging(
+            model=mock_model, algorithm=federated_algorithm_with_model
+        )
+        protocol.run(
             pod_identifiers=pod_identifiers,
-            model=mock_model,
-            algorithm=federated_algorithm_with_model,
             private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
         )
 
-        mock_modeller_run_method.assert_called_once_with(pod_identifiers)
+        mock_modeller_run_method.assert_called_once_with(
+            pod_identifiers, require_all_pods=False
+        )
         mock_model.assert_not_called()
         mock_model.backend_tensor_shim.assert_not_called()
         assert "Ignoring provided model. Algorithm already has a model." in caplog.text
