@@ -341,6 +341,8 @@ class TestPyTorchBitfountModel:
         )
 
         assert preds is not None
+        assert isinstance(preds, list)
+        assert isinstance(preds[0], np.ndarray)
         assert len(preds) == len(dataset.get_dataset_split(DataSplit.TEST))
         assert inference_model.n_classes == len(preds[0])
 
@@ -491,17 +493,21 @@ class TestPyTorchBitfountModel:
             filename="DummyModel.py", extra_imports=extra_imports
         )
 
+    @pytest.mark.parametrize(
+        "method_name", ["_fit_federated", "_evaluate_federated", "_predict_federated"]
+    )
     @unit_test
-    def test__fit_federated(
+    def test_federated_sugar_method(
         self,
         datastructure: DataStructure,
         dummy_model_class: Type[PyTorchBitfountModel],
+        method_name: str,
         mock_bitfount_session: Mock,
         mocker: MockerFixture,
     ) -> None:
-        """Tests private _fit_federated method with a custom model.
+        """Tests private _{fit,evaluate,predict}_federated methods with a custom model.
 
-        Checks that DistributedModelMixIn._fit_federated helper method creates
+        Checks that the sugar method creates
         correct instances and runs the modeller correctly.
         """
         # change datastructure table to be a mapping
@@ -513,7 +519,6 @@ class TestPyTorchBitfountModel:
         model = dummy_model_class(
             datastructure=datastructure, schema=create_schema(classification=True)
         )
-
         # Mock the serialization of the model
         mock_serialize_method = mocker.patch.object(
             model,
@@ -527,33 +532,37 @@ class TestPyTorchBitfountModel:
 
         # Run method
         pod_identifiers = ["bitfount/census-income", "bitfount/census-income-2"]
-        model._fit_federated(
+        method_under_test = getattr(model, method_name)
+        method_under_test(
             pod_identifiers=pod_identifiers,
             private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
         )
-
         # Check run method was called correctly
         # TODO: [BIT-983] Should this check that the Modeller was instantiated
         #       correctly? Related to whether we should mock out the helper calls.
         mock_modeller_run_method.assert_called_once_with(
-            pod_identifiers, require_all_pods=False, model_out=None
+            pod_identifiers, require_all_pods=False, model_out=None, project_id=None
         )
         mock_serialize_method.assert_called_once_with(
             filename="DummyModel.py", extra_imports=None
         )
 
+    @pytest.mark.parametrize(
+        "method_name", ["_fit_federated", "_evaluate_federated", "_predict_federated"]
+    )
     @unit_test
-    def test__fit_federated_raises_error(
+    def test_federated_sugar_method_raises_error(
         self,
         datastructure: DataStructure,
         dummy_model_class: Type[PyTorchBitfountModel],
+        method_name: str,
         mock_bitfount_session: Mock,
         mocker: MockerFixture,
     ) -> None:
-        """Tests private _fit_federated method with a custom model.
+        """Tests private _{fit,evaluate,predict}_federated raises a ValueError.
 
-        Checks that DistributedModelMixIn._fit_federated helper method creates
-        correct instances and runs the modeller correctly.
+        Checks that the DataStructure table name as a string
+        raises error when training on multiple pods.
         """
         # Create model to test
         model = dummy_model_class(
@@ -567,8 +576,9 @@ class TestPyTorchBitfountModel:
         )
         # Run method
         pod_identifiers = ["bitfount/census-income", "bitfount/census-income-2"]
+        method_under_test = getattr(model, method_name)
         with pytest.raises(ValueError):
-            model._fit_federated(
+            method_under_test(
                 pod_identifiers=pod_identifiers,
                 private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
             )
@@ -623,90 +633,6 @@ class TestPyTorchBitfountModel:
         model.initialise_model()
         assert model.input_size is not None
 
-    @unit_test
-    def test__evaluate_federated(
-        self,
-        datastructure: DataStructure,
-        dummy_model_class: Type[PyTorchBitfountModel],
-        mock_bitfount_session: Mock,
-        mocker: MockerFixture,
-    ) -> None:
-        """Tests private _evaluate_federated method with a custom model.
-
-        Checks that DistributedModelMixIn._fit_federated helper method creates
-        correct instances and runs the modeller correctly.
-        """
-        datastructure.table = {"pod1": "pod1_table", "pod2": "pod2_table"}
-        # Create model to test
-        model = dummy_model_class(
-            datastructure=datastructure, schema=create_schema(classification=True)
-        )
-
-        # Mock the serialization of the model
-        mock_serialize_method = mocker.patch.object(
-            model,
-            "serialize_model_source_code",
-            return_value="This is some source code",
-        )
-
-        # Patch out the modeller's run method as we only care how it is called
-        # from _fit_federated.
-        mock_modeller_run_method = mocker.patch.object(_Modeller, "run")
-
-        # Run method
-        pod_identifiers = ["test_username/pod1", "test_username/pod2"]
-        model._evaluate_federated(
-            pod_identifiers=pod_identifiers,
-            private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
-        )
-
-        # Check run method was called correctly
-        # TODO: [BIT-983] Should this check that the Modeller was instantiated
-        #       correctly? Related to whether we should mock out the helper calls.
-        mock_modeller_run_method.assert_called_once_with(
-            pod_identifiers, require_all_pods=False, model_out=None
-        )
-        mock_serialize_method.assert_called_once_with(
-            filename="DummyModel.py", extra_imports=None
-        )
-
-    @unit_test
-    def test__evaluate_federated_raises_error(
-        self,
-        datastructure: DataStructure,
-        dummy_model_class: Type[PyTorchBitfountModel],
-        mock_bitfount_session: Mock,
-        mocker: MockerFixture,
-    ) -> None:
-        """Tests private _evaluate_federated method with a custom model.
-
-        Checks that DistributedModelMixIn._fit_federated helper method creates
-        correct instances and runs the modeller correctly.
-        """
-        # Create model to test
-        model = dummy_model_class(
-            datastructure=datastructure, schema=create_schema(classification=True)
-        )
-
-        # Mock the serialization of the model
-        mocker.patch.object(
-            model,
-            "serialize_model_source_code",
-            return_value="This is some source code",
-        )
-
-        # Patch out the modeller's run method as we only care how it is called
-        # from _fit_federated.
-        mocker.patch.object(_Modeller, "run")
-
-        # Run method
-        pod_identifiers = ["bitfount/census-income", "bitfount/census-income-2"]
-        with pytest.raises(ValueError):
-            model._evaluate_federated(
-                pod_identifiers=pod_identifiers,
-                private_key_or_file=TEST_SECURITY_FILES / "test_private.testkey",
-            )
-
 
 @backend_test
 class TestUNetPyTorchBitfountModel:
