@@ -9,16 +9,18 @@ import pandas as pd
 import pytest
 import torch
 
-from bitfount import (
-    BitfountSchema,
-    FederatedAveraging,
-    FederatedModelTraining,
-    Optimizer,
-    Pod,
-)
 from bitfount.backends.pytorch import PyTorchTabularClassifier
 from bitfount.data.datasources.dataframe_source import DataFrameSource
+from bitfount.data.schema import BitfountSchema
+from bitfount.federated.algorithms.model_algorithms.federated_training import (
+    FederatedModelTraining,
+)
 from bitfount.federated.helper import _create_aggregator
+from bitfount.federated.pod import Pod
+from bitfount.federated.protocols.model_protocols.federated_averaging import (
+    FederatedAveraging,
+)
+from bitfount.models.base_models import Optimizer
 from bitfount.runners.config_schemas import PodDataConfig
 from tests.utils.helper import (
     backend_test,
