@@ -1,6 +1,6 @@
 """Test dataset classes in data/datasets.py."""
 import logging
-from typing import Iterator, Optional, Tuple
+from typing import Iterator, List, Optional, Tuple
 from unittest.mock import Mock
 
 from PIL import Image
@@ -13,6 +13,7 @@ from pytest_mock import MockerFixture
 import sqlalchemy
 
 from bitfount.data.datasets import _BitfountDataset, _IterableBitfountDataset
+from bitfount.data.datasources.base_source import FileSystemIterableSource
 from bitfount.data.datasources.database_source import DatabaseSource
 from bitfount.data.datasources.dataframe_source import DataFrameSource
 from bitfount.data.datasplitters import (
@@ -67,41 +68,51 @@ class FakeSplitter(DatasetSplitter):
     def get_split_query(self, datasource: DatabaseSource, split: DataSplit) -> str:
         """Returns query for given split."""
 
+    def get_filenames(
+        self, datasource: FileSystemIterableSource, split: DataSplit
+    ) -> List[str]:
+        """Returns filenames for given split."""
 
-class TestBaseBitfountDataset:
+
+@fixture
+def test_percentage() -> int:
+    """Test percentage value."""
+    return 25
+
+
+@fixture
+def validation_percentage() -> int:
+    """Validation percentage value."""
+    return 55
+
+
+@fixture
+def train_percentage(test_percentage: int, validation_percentage: int) -> int:
+    """Train percentage value."""
+    return (100 - test_percentage) - validation_percentage
+
+
+@fixture
+def target() -> str:
+    """Target value."""
+    return "TARGET"
+
+
+@fixture
+def fake_splitter(
+    test_percentage: int, train_percentage: int, validation_percentage: int
+) -> FakeSplitter:
+    """Fake splitter defined using train/validation/test values."""
+    return FakeSplitter(
+        np.array(range(int(DATASET_ROW_COUNT * (train_percentage / 100)))),
+        np.array(range(int(DATASET_ROW_COUNT * (validation_percentage / 100)))),
+        np.array(range(int(DATASET_ROW_COUNT * (test_percentage / 100)))),
+    )
+
+
+class TestBitfountDataset:
     """Tests BaseBitfountDataset class."""
 
-    @fixture
-    def test_percentage(self) -> int:
-        """Test percentage value."""
-        return 25
-
-    @fixture
-    def validation_percentage(self) -> int:
-        """Validation percentage value."""
-        return 55
-
-    @fixture
-    def train_percentage(self, test_percentage: int, validation_percentage: int) -> int:
-        """Train percentage value."""
-        return (100 - test_percentage) - validation_percentage
-
-    @fixture
-    def target(self) -> str:
-        """Target value."""
-        return "TARGET"
-
-    @fixture
-    def fake_splitter(
-        self, test_percentage: int, train_percentage: int, validation_percentage: int
-    ) -> FakeSplitter:
-        """Fake splitter defined using train/validation/test values."""
-        return FakeSplitter(
-            np.array(range(int(DATASET_ROW_COUNT * (train_percentage / 100)))),
-            np.array(range(int(DATASET_ROW_COUNT * (validation_percentage / 100)))),
-            np.array(range(int(DATASET_ROW_COUNT * (test_percentage / 100)))),
-        )
-
     @fixture
     def datastructure(self, target: str) -> DataStructure:
         """Datastructure fixture."""
@@ -131,7 +142,7 @@ class TestBaseBitfountDataset:
 
     @unit_test
     def test_load_grayscale_image(
-        self, target: str, grayscale_image_dataframe: pd.DataFrame
+        self, grayscale_image_dataframe: pd.DataFrame, target: str
     ) -> None:
         """Test _load_image method."""
         datasource = DataFrameSource(grayscale_image_dataframe)
@@ -248,22 +259,6 @@ class TestBaseBitfountDataset:
             in caplog.text
         )
 
-    @unit_test
-    def test_datasource_data_split_called_twice(
-        self, caplog: LogCaptureFixture, iter_tabular_dataset: _IterableBitfountDataset
-    ) -> None:
-        """Tests that the log is printed if split_data called twice."""
-        caplog.set_level(logging.DEBUG)
-
-        assert iter_tabular_dataset.datasource._data_is_split is False
-
-        iter_tabular_dataset._split_data()
-
-        assert iter_tabular_dataset.datasource._data_is_split is True
-        # split data in DataBunch into training, validation, test sets
-        iter_tabular_dataset._split_data()
-        assert "Data is already split, keeping the current split." in caplog.text
-
     @pytest.mark.parametrize(
         "datasplit_type", [DataSplit.TEST, DataSplit.TRAIN, DataSplit.VALIDATION]
     )
@@ -319,63 +314,6 @@ class TestBaseBitfountDataset:
             == dataset.get_dataset_split(datasplit_type).shape[0]
         )
 
-    @unit_test
-    @pytest.mark.parametrize(
-        "datasource_splitter, datastructure_splitter",
-        [
-            (PercentageSplitter(), PercentageSplitter()),
-            (PercentageSplitter(), None),
-            (None, PercentageSplitter()),
-            (None, None),
-        ],
-    )
-    def test_resolve_data_splitter(
-        self,
-        dataframe: pd.DataFrame,
-        datasource_splitter: Optional[DatasetSplitter],
-        datastructure_splitter: Optional[DatasetSplitter],
-    ) -> None:
-        """Checks data splitter is resolved correctly.
-
-        If datasource has a data_splitter, use it to split the data.
-        Else if the datastructure has a data_splitter use that splitter.
-        Else use the PercentageSplitter.
-        """
-        datasource = DataFrameSource(dataframe, data_splitter=datasource_splitter)
-        datasource.load_data()
-        target = "TARGET"
-
-        schema = BitfountSchema()
-        schema.add_datasource_tables(datasource, table_name=TABLE_NAME)
-        datasource.data = datasource.data.drop(
-            columns=schema.get_feature_names(TABLE_NAME, SemanticType.TEXT)
-        )
-        datastructure = DataStructure(
-            target=target, ignore_cols=["image"], table=TABLE_NAME
-        )
-        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
-        dataset = _IterableBitfountDataset(
-            datasource=datasource,
-            target=target,
-            selected_cols_semantic_types=datastructure.selected_cols_w_types,
-            selected_cols=datastructure.selected_cols,
-            data_split=DataSplit.TRAIN,
-            schema=schema.get_table_schema(TABLE_NAME),
-        )
-        resolved_splitter = dataset._resolve_data_splitter(datastructure_splitter)
-        assert isinstance(resolved_splitter, PercentageSplitter)
-
-        if datasource_splitter:
-            assert datasource.data_splitter is not None
-            assert resolved_splitter is datasource.data_splitter
-            assert resolved_splitter is not datastructure_splitter
-        elif datastructure_splitter:
-            assert datasource.data_splitter is None
-            assert resolved_splitter is datastructure_splitter
-        else:
-            assert resolved_splitter is not datastructure_splitter
-            assert resolved_splitter is not datasource_splitter
-
     @unit_test
     def test_zero_validation_test_size(
         self, dataframe: pd.DataFrame, target: str
@@ -451,198 +389,24 @@ class TestBaseBitfountDataset:
         ) == len(datasource.data)
 
     @unit_test
-    def test_training_set_unloaded_raises_error(
-        self, mock_engine: Mock, mocker: MockerFixture
-    ) -> None:
-        """Tests that calling the `training_set` property raises an error.
-
-        The error should be a `DataNotLoadedError` when the data is multi-table because
-        it is never loaded.
-        """
-        target = "TARGET"
-        table_name = "dummy_data"
-        datastructure = DataStructure(
-            target=target, ignore_cols=["image"], table=table_name
-        )
-
-        db_conn = DatabaseConnection(
-            mock_engine, table_names=["dummy_data", "dummy_data_2"]
-        )
-        datasource = DatabaseSource(db_conn, seed=420)
-        mock_dtypes = {"col": np.dtype(int)}
-        mocker.patch.object(datasource, "get_dtypes", return_value=mock_dtypes)
-        datasource._table_hashes.add(_generate_dtypes_hash(mock_dtypes))
-        datasource.load_data()
-        schema = BitfountSchema()
-        schema.add_datasource_tables(datasource, table_name=table_name)
-        # Using IterableBitfountDataset but get_dataset_split is a
-        # concrete method of _BaseBitfountDataset
-        db_dataset = _IterableBitfountDataset(
-            datasource=datasource,
-            target=target,
-            selected_cols=datastructure.selected_cols,
-            data_split=DataSplit.TRAIN,
-            schema=schema.get_table_schema(table_name),
-            selected_cols_semantic_types=datastructure.selected_cols_w_types,
-        )
-        with pytest.raises(DataNotLoadedError):
-            db_dataset.get_dataset_split(DataSplit.TRAIN)
-
-    @unit_test
-    def test_datasource_data_split_flag_updated(self, dataframe: pd.DataFrame) -> None:
-        """Tests that the data_is_split flag is updated."""
-        target = "TARGET"
-        ds = DataFrameSource(dataframe, selected_cols=["M", "F"])
-        ds.load_data()
-        assert ds._data_is_split is False
-
-        schema = BitfountSchema()
-        schema.add_datasource_tables(ds, table_name=TABLE_NAME)
-        datastructure = DataStructure(
-            target=target, ignore_cols=["image"], table=TABLE_NAME
-        )
-        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
-        dataset = _IterableBitfountDataset(
-            datasource=ds,
-            selected_cols_semantic_types=datastructure.selected_cols_w_types,
-            selected_cols=datastructure.selected_cols,
-            data_split=DataSplit.TRAIN,
-            schema=schema.get_table_schema(TABLE_NAME),
-        )
-        dataset._split_data()
-        assert ds._data_is_split is True
-
-    @unit_test
-    def test_get_dataset_split_length_returns_length_correctly(
-        self, dataframe: pd.DataFrame, target: str
-    ) -> None:
-        """Tests that `get_dataset_split_length` returns the correct length.
-
-        This is done with the `SplitterDefinedInData` splitter.
-        """
-        dataframe["BITFOUNT_SPLIT_CATEGORY"] = np.random.choice(
-            ["TRAIN", "VALIDATE", "TEST"], size=dataframe.shape[0]
-        )
-
-        ds = DataFrameSource(dataframe, data_splitter=SplitterDefinedInData())
-        ds.load_data()
-        schema = BitfountSchema()
-        schema.add_datasource_tables(ds, table_name=TABLE_NAME)
-        datastructure = DataStructure(
-            target=target, ignore_cols=["image"], table=TABLE_NAME
-        )
-        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
-        dataset = _BitfountDataset(
-            datasource=ds,
-            target=target,
-            selected_cols_semantic_types=datastructure.selected_cols_w_types,
-            selected_cols=datastructure.selected_cols,
-            data_split=DataSplit.TRAIN,
-            schema=schema.get_table_schema(TABLE_NAME),
-        )
-        train_length = dataset.get_dataset_split_length(DataSplit.TRAIN)
-        assert train_length == len(
-            dataframe.loc[dataframe["BITFOUNT_SPLIT_CATEGORY"] == "TRAIN"]
-        )
-
-    @unit_test
-    def test_get_dataset_split_length_executes_query(
-        self,
-        mock_engine: Mock,
-        mocker: MockerFixture,
-    ) -> None:
-        """Tests that `get_dataset_split_length` executes query with database."""
-        db_conn = DatabaseConnection(
-            mock_engine, table_names=["dummy_data", "dummy_data_2"]
-        )
-        datasource = DatabaseSource(
-            db_conn,
-            data_splitter=SplitterDefinedInData(
-                column_name='q."M"', training_set_label="True"
-            ),
-        )
-        datasource.load_data(sql_query="SELECT * FROM dummy_data")
-        mock_connect = mocker.patch.object(datasource.con, "connect")
-        mock_connect.__enter__.return_value = Mock()
-        mocker.patch.object(_IterableBitfountDataset, "_set_column_name_attributes")
-        dataset = _IterableBitfountDataset(datasource, Mock(), Mock(), Mock(), Mock())
-        dataset.get_dataset_split_length(DataSplit.TRAIN)
-
-        mock_connect.assert_called_once()
-
-    @integration_test
-    def test_get_dataset_split_length_with_multitable_database_connection(
-        self,
-        dataframe: pd.DataFrame,
-        db_session: sqlalchemy.engine.base.Engine,
-        mocker: MockerFixture,
-    ) -> None:
-        """Tests that `get_dataset_split_length` returns the correct length."""
-        db_conn = DatabaseConnection(
-            db_session, table_names=["dummy_data", "dummy_data_2"]
-        )
-        datasource = DatabaseSource(
-            db_conn,
-            data_splitter=SplitterDefinedInData(
-                column_name='q."M"', training_set_label="True"
-            ),
-        )
-        datasource.load_data(sql_query="SELECT * FROM dummy_data")
-        mocker.patch.object(_IterableBitfountDataset, "_set_column_name_attributes")
-        dataset = _IterableBitfountDataset(datasource, Mock(), Mock(), Mock(), Mock())
-        train_length = dataset.get_dataset_split_length(DataSplit.TRAIN)
-        assert train_length == len(dataframe.loc[dataframe["M"] == True])  # noqa: E712
-
-    @unit_test
-    def test_unable_get_dataset_split_length(
-        self, dataframe: pd.DataFrame, fake_splitter: FakeSplitter, target: str
-    ) -> None:
-        """Test error raise when unable to split dataset."""
-        ds = DataFrameSource(dataframe, data_splitter=fake_splitter)
-        ds.load_data()
-
-        schema = BitfountSchema()
-        schema.add_datasource_tables(ds, table_name=TABLE_NAME)
-        datastructure = DataStructure(
-            target=target, ignore_cols=["image"], table=TABLE_NAME
-        )
-        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
-
-        dataset = _BitfountDataset(
-            datasource=ds,
-            target=target,
-            selected_cols_semantic_types=datastructure.selected_cols_w_types,
-            selected_cols=datastructure.selected_cols,
-            data_split=DataSplit.TRAIN,
-            schema=schema.get_table_schema(TABLE_NAME),
-        )
-        dataset.datasource._data_is_loaded = False
-        with pytest.raises(
-            DataNotLoadedError,
-            match="Unable to get length of dataset split",
-        ):
-            dataset.get_dataset_split_length(DataSplit.TRAIN)
-
-
-@unit_test
-class TestBitfountDataset:
-    """Tests BitfountDataset class."""
-
     def test_len_tab_data(self, tabular_dataset: _BitfountDataset) -> None:
         """Tests tabular dataset __len__ method."""
         assert tabular_dataset.datasource._train_idxs is not None
         assert len(tabular_dataset) == len(tabular_dataset.datasource._train_idxs)
 
+    @unit_test
     def test_len_img_data(self, image_tab_dataset: _BitfountDataset) -> None:
         """Tests image dataset __len__ method."""
         assert image_tab_dataset.datasource._train_idxs is not None
         assert len(image_tab_dataset) == len(image_tab_dataset.datasource._train_idxs)
 
+    @unit_test
     def test_len_img_tab_data(self, image_dataset: _BitfountDataset) -> None:
         """Tests dataset __len__ method."""
         assert image_dataset.datasource._train_idxs is not None
         assert len(image_dataset) == len(image_dataset.datasource._train_idxs)
 
+    @unit_test
     def test_len_multiimg_data(self, multiimage_dataset: _BitfountDataset) -> None:
         """Tests multi-image dataset __len__ method."""
         assert multiimage_dataset.datasource._train_idxs is not None
@@ -812,17 +576,6 @@ class TestIterableBitfountDataset:
 
         dataset.yield_dataset_split.assert_called_once()  # type: ignore[attr-defined] # Reason: check if mocked method called. # noqa: B950
 
-    @unit_test
-    def test_yield_dataset_split_raises_value_error_if_not_database_loader(
-        self, iter_tabular_dataset: _IterableBitfountDataset
-    ) -> None:
-        """Tests that ValueError is raised if loader is not a DatabaseSource."""
-        with pytest.raises(
-            ValueError,
-            match="Iterating over in-memory data is currently not supported.",
-        ):
-            next(iter_tabular_dataset.yield_dataset_split(DataSplit.TRAIN))
-
     @integration_test
     def test_yield_dataset_split_raises_value_error_if_no_query_or_table_name_provided(
         self,
@@ -912,7 +665,7 @@ class TestIterableBitfountDataset:
 
         database_partition_size = 10
         datasource = DatabaseSource(
-            db_conn, seed=420, database_partition_size=database_partition_size
+            db_conn, seed=420, partition_size=database_partition_size
         )
 
         # Test when load_data is called without query
@@ -990,7 +743,7 @@ class TestIterableBitfountDataset:
         assert not datasource._data_is_loaded
         assert isinstance(datasource, DatabaseSource)
         expected_output = pd.read_sql(
-            f"{query} LIMIT {datasource.database_partition_size}",
+            f"{query} LIMIT {datasource.partition_size}",
             con=db_conn.con,
         )
         mocker.patch.object(_IterableBitfountDataset, "_set_column_name_attributes")
@@ -1077,3 +830,215 @@ class TestIterableBitfountDataset:
         assert len(datasource._train_idxs) + len(datasource._validation_idxs) + len(
             datasource._test_idxs
         ) == len(datasource.data)
+
+    @unit_test
+    def test_datasource_data_split_called_twice(
+        self, caplog: LogCaptureFixture, iter_tabular_dataset: _IterableBitfountDataset
+    ) -> None:
+        """Tests that the log is printed if split_data called twice."""
+        caplog.set_level(logging.DEBUG)
+
+        assert iter_tabular_dataset.datasource._data_is_split is False
+
+        iter_tabular_dataset._split_data()
+
+        assert iter_tabular_dataset.datasource._data_is_split is True
+        # split data in DataBunch into training, validation, test sets
+        iter_tabular_dataset._split_data()
+        assert "Data is already split, keeping the current split." in caplog.text
+
+    @unit_test
+    @pytest.mark.parametrize(
+        "datasource_splitter, datastructure_splitter",
+        [
+            (PercentageSplitter(), PercentageSplitter()),
+            (PercentageSplitter(), None),
+            (None, PercentageSplitter()),
+            (None, None),
+        ],
+    )
+    def test_resolve_data_splitter(
+        self,
+        dataframe: pd.DataFrame,
+        datasource_splitter: Optional[DatasetSplitter],
+        datastructure_splitter: Optional[DatasetSplitter],
+    ) -> None:
+        """Checks data splitter is resolved correctly.
+
+        If datasource has a data_splitter, use it to split the data.
+        Else if the datastructure has a data_splitter use that splitter.
+        Else use the PercentageSplitter.
+        """
+        datasource = DataFrameSource(dataframe, data_splitter=datasource_splitter)
+        datasource.load_data()
+        target = "TARGET"
+
+        schema = BitfountSchema()
+        schema.add_datasource_tables(datasource, table_name=TABLE_NAME)
+        datasource.data = datasource.data.drop(
+            columns=schema.get_feature_names(TABLE_NAME, SemanticType.TEXT)
+        )
+        datastructure = DataStructure(
+            target=target, ignore_cols=["image"], table=TABLE_NAME
+        )
+        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
+        dataset = _IterableBitfountDataset(
+            datasource=datasource,
+            target=target,
+            selected_cols_semantic_types=datastructure.selected_cols_w_types,
+            selected_cols=datastructure.selected_cols,
+            data_split=DataSplit.TRAIN,
+            schema=schema.get_table_schema(TABLE_NAME),
+        )
+        resolved_splitter = dataset._resolve_data_splitter(datastructure_splitter)
+        assert isinstance(resolved_splitter, PercentageSplitter)
+
+        if datasource_splitter:
+            assert datasource.data_splitter is not None
+            assert resolved_splitter is datasource.data_splitter
+            assert resolved_splitter is not datastructure_splitter
+        elif datastructure_splitter:
+            assert datasource.data_splitter is None
+            assert resolved_splitter is datastructure_splitter
+        else:
+            assert resolved_splitter is not datastructure_splitter
+            assert resolved_splitter is not datasource_splitter
+
+    @unit_test
+    def test_training_set_unloaded_raises_error(
+        self, mock_engine: Mock, mocker: MockerFixture
+    ) -> None:
+        """Tests that calling the `training_set` property raises an error.
+
+        The error should be a `DataNotLoadedError` when the data is multi-table because
+        it is never loaded.
+        """
+        target = "TARGET"
+        table_name = "dummy_data"
+        datastructure = DataStructure(
+            target=target, ignore_cols=["image"], table=table_name
+        )
+
+        db_conn = DatabaseConnection(
+            mock_engine, table_names=["dummy_data", "dummy_data_2"]
+        )
+        datasource = DatabaseSource(db_conn, seed=420)
+        mock_dtypes = {"col": np.dtype(int)}
+        mocker.patch.object(datasource, "get_dtypes", return_value=mock_dtypes)
+        datasource._table_hashes.add(_generate_dtypes_hash(mock_dtypes))
+        datasource.load_data()
+        schema = BitfountSchema()
+        schema.add_datasource_tables(datasource, table_name=table_name)
+        # Using IterableBitfountDataset but get_dataset_split is a
+        # concrete method of _BaseBitfountDataset
+        db_dataset = _IterableBitfountDataset(
+            datasource=datasource,
+            target=target,
+            selected_cols=datastructure.selected_cols,
+            data_split=DataSplit.TRAIN,
+            schema=schema.get_table_schema(table_name),
+            selected_cols_semantic_types=datastructure.selected_cols_w_types,
+        )
+        with pytest.raises(DataNotLoadedError):
+            db_dataset.get_dataset_split(DataSplit.TRAIN)
+
+    @unit_test
+    def test_datasource_data_split_flag_updated(self, dataframe: pd.DataFrame) -> None:
+        """Tests that the data_is_split flag is updated."""
+        target = "TARGET"
+        ds = DataFrameSource(dataframe, selected_cols=["M", "F"])
+        ds.load_data()
+        assert ds._data_is_split is False
+
+        schema = BitfountSchema()
+        schema.add_datasource_tables(ds, table_name=TABLE_NAME)
+        datastructure = DataStructure(
+            target=target, ignore_cols=["image"], table=TABLE_NAME
+        )
+        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
+        dataset = _IterableBitfountDataset(
+            datasource=ds,
+            selected_cols_semantic_types=datastructure.selected_cols_w_types,
+            selected_cols=datastructure.selected_cols,
+            data_split=DataSplit.TRAIN,
+            schema=schema.get_table_schema(TABLE_NAME),
+        )
+        dataset._split_data()
+        assert ds._data_is_split is True
+
+    @unit_test
+    def test_get_dataset_split_length_executes_query(
+        self,
+        mock_engine: Mock,
+        mocker: MockerFixture,
+    ) -> None:
+        """Tests that `get_dataset_split_length` executes query with database."""
+        db_conn = DatabaseConnection(
+            mock_engine, table_names=["dummy_data", "dummy_data_2"]
+        )
+        datasource = DatabaseSource(
+            db_conn,
+            data_splitter=SplitterDefinedInData(
+                column_name='q."M"', training_set_label="True"
+            ),
+        )
+        datasource.load_data(sql_query="SELECT * FROM dummy_data")
+        mock_connect = mocker.patch.object(datasource.con, "connect")
+        mock_connect.__enter__.return_value = Mock()
+        mocker.patch.object(_IterableBitfountDataset, "_set_column_name_attributes")
+        dataset = _IterableBitfountDataset(datasource, Mock(), Mock(), Mock(), Mock())
+        dataset.get_dataset_split_length(DataSplit.TRAIN)
+
+        mock_connect.assert_called_once()
+
+    @integration_test
+    def test_get_dataset_split_length_with_multitable_database_connection(
+        self,
+        dataframe: pd.DataFrame,
+        db_session: sqlalchemy.engine.base.Engine,
+        mocker: MockerFixture,
+    ) -> None:
+        """Tests that `get_dataset_split_length` returns the correct length."""
+        db_conn = DatabaseConnection(
+            db_session, table_names=["dummy_data", "dummy_data_2"]
+        )
+        datasource = DatabaseSource(
+            db_conn,
+            data_splitter=SplitterDefinedInData(
+                column_name='q."M"', training_set_label="True"
+            ),
+        )
+        datasource.load_data(sql_query="SELECT * FROM dummy_data")
+        mocker.patch.object(_IterableBitfountDataset, "_set_column_name_attributes")
+        dataset = _IterableBitfountDataset(datasource, Mock(), Mock(), Mock(), Mock())
+        train_length = dataset.get_dataset_split_length(DataSplit.TRAIN)
+        assert train_length == len(dataframe.loc[dataframe["M"] == True])  # noqa: E712
+
+    @unit_test
+    def test_unable_to_get_dataset_split_length(
+        self, dataframe: pd.DataFrame, fake_splitter: FakeSplitter, target: str
+    ) -> None:
+        """Test error raise when unable to split dataset."""
+        ds = DataFrameSource(dataframe, data_splitter=fake_splitter)
+        ds.load_data()
+
+        schema = BitfountSchema()
+        schema.add_datasource_tables(ds, table_name=TABLE_NAME)
+        datastructure = DataStructure(
+            target=target, ignore_cols=["image"], table=TABLE_NAME
+        )
+        datastructure.set_training_column_split_by_semantic_type(schema.tables[0])
+
+        dataset = _IterableBitfountDataset(
+            datasource=ds,
+            target=target,
+            selected_cols_semantic_types=datastructure.selected_cols_w_types,
+            selected_cols=datastructure.selected_cols,
+            data_split=DataSplit.TRAIN,
+            schema=schema.get_table_schema(TABLE_NAME),
+        )
+        with pytest.raises(
+            DataNotLoadedError,
+            match="Unable to get length of dataset split",
+        ):
+            dataset.get_dataset_split_length(DataSplit.TRAIN)
