@@ -79,6 +79,7 @@ class TestDataStructure:
         transform = DataStructureTransformConfig(
             dataset=[{"convert_to": {"type": "string", "col": "TARGET"}}],
             batch=[{"albumentations": {"col": ["image"], "keep_original": "image"}}],
+            auto_convert_grayscale_images=False,
         )
         table_config = DataStructureTableConfig(table=TABLE_NAME)
         ds = DataStructure.create_datastructure(
@@ -98,6 +99,7 @@ class TestDataStructure:
         assert ds.dataset_transforms == [
             {"convert_to": {"type": "string", "col": "TARGET"}}
         ]
+        assert not ds.auto_convert_grayscale_images
 
     def test_create_datastructure_select_config_raises_error(self) -> None:
         """Tests that the providing both include and exclude cols raises error."""
