@@ -44,11 +44,11 @@ from bitfount.federated.types import (
     _TaskRequestMessageGenerator,
 )
 from bitfount.hub.api import BitfountAM, BitfountHub
-from bitfount.hub.authentication_flow import (
+from bitfount.hub.authentication_flow import _AuthEnv
+from bitfount.hub.authentication_handlers import (
     _AUTHORIZATION_PENDING_ERROR,
     _DEVICE_CODE_GRANT_TYPE,
     _SLOW_DOWN_ERROR,
-    _AuthEnv,
 )
 from bitfount.hub.types import (
     _DeviceAccessTokenRequestDict,
@@ -365,6 +365,7 @@ class TestLocalAuthorisation:
         mock_task_request_import: Mock,
         mock_task_request_message: NonCallableMock,
         mock_task_request_message_import: Mock,
+        opt_project_id: Optional[str],
         pod_identifiers: List[str],
         serialized_protocol: SerializedProtocol,
     ) -> None:
@@ -374,6 +375,7 @@ class TestLocalAuthorisation:
             pod_identifiers=pod_identifiers,
             aes_key=aes_key,
             pod_public_key=mock_pod_public_key,
+            project_id=opt_project_id,
         )
 
         # Check TaskRequest constructed as expected
@@ -395,6 +397,7 @@ class TestLocalAuthorisation:
             serialized_protocol,
             IdentityVerificationMethod.LOCAL.value,
             mock_encrypted_task_request.serialize.return_value,
+            opt_project_id,
         )
         # Check that output value is serialized TaskRequestMessage
         assert trm == mock_task_request_message.serialize.return_value
@@ -512,9 +515,7 @@ class TestSignatureBasedAuthorisation:
         # Check API call made correctly
         mock_access_manager.check_signature_based_access_request.assert_called_once_with(
             unsigned_task=encrypted_task,
-            signed_task=_SignedEncryptedTaskRequest(
-                encrypted_task, signature
-            ).serialize(),
+            task_signature=signature,
             pod_identifier=pod_identifier,
             serialized_protocol=serialized_protocol,
             modeller_name=modeller_name,
@@ -535,6 +536,7 @@ class TestSignatureBasedAuthorisation:
         mock_task_request_import: Mock,
         mock_task_request_message: NonCallableMock,
         mock_task_request_message_import: Mock,
+        opt_project_id: Optional[str],
         pod_identifiers: List[str],
         serialized_protocol: SerializedProtocol,
     ) -> None:
@@ -550,6 +552,7 @@ class TestSignatureBasedAuthorisation:
             pod_identifiers=pod_identifiers,
             aes_key=aes_key,
             pod_public_key=mock_pod_public_key,
+            project_id=opt_project_id,
         )
 
         # Check TaskRequest constructed as expected
@@ -578,6 +581,7 @@ class TestSignatureBasedAuthorisation:
             serialized_protocol,
             IdentityVerificationMethod.KEYS.value,
             mock_signed_encrypted_task_request.serialize.return_value,
+            opt_project_id,
         )
         # Check that output value is serialized TaskRequestMessage
         assert trm == mock_task_request_message.serialize.return_value
@@ -766,6 +770,16 @@ class TestSAMLAuthorisation:
         assert extracted == encrypted_task_request
 
 
+@fixture(params=(True, False), ids=("project_id_incl", "no_project_id"))
+def opt_project_id(request: PytestRequest) -> Optional[str]:
+    """Project ID."""
+    incl_project_id: bool = request.param
+    if incl_project_id:
+        return "this-is-a-project-id"
+    else:
+        return None
+
+
 @fixture
 def mock_access_manager() -> Mock:
     """Mock access manager."""
@@ -858,6 +872,7 @@ class TestOIDCAuthCodeAuthorisation:
         mock_task_request_import: Mock,
         mock_task_request_message: NonCallableMock,
         mock_task_request_message_import: Mock,
+        opt_project_id: Optional[str],
         pod_identifiers: List[str],
         serialized_protocol: SerializedProtocol,
     ) -> None:
@@ -867,6 +882,7 @@ class TestOIDCAuthCodeAuthorisation:
             pod_identifiers=pod_identifiers,
             aes_key=aes_key,
             pod_public_key=mock_pod_public_key,
+            project_id=opt_project_id,
         )
 
         # Check TaskRequest constructed as expected
@@ -888,6 +904,7 @@ class TestOIDCAuthCodeAuthorisation:
             serialized_protocol,
             IdentityVerificationMethod.OIDC_ACF_PKCE.value,
             mock_encrypted_task_request.serialize.return_value,
+            opt_project_id,
         )
         # Check that output value is serialized TaskRequestMessage
         assert trm == mock_task_request_message.serialize.return_value
@@ -1116,6 +1133,7 @@ class TestOIDCDeviceCodeAuthorisation:
         mock_task_request_import: Mock,
         mock_task_request_message: NonCallableMock,
         mock_task_request_message_import: Mock,
+        opt_project_id: Optional[str],
         pod_identifiers: List[str],
         serialized_protocol: SerializedProtocol,
     ) -> None:
@@ -1125,6 +1143,7 @@ class TestOIDCDeviceCodeAuthorisation:
             pod_identifiers=pod_identifiers,
             aes_key=aes_key,
             pod_public_key=mock_pod_public_key,
+            project_id=opt_project_id,
         )
 
         # Check TaskRequest constructed as expected
@@ -1146,6 +1165,7 @@ class TestOIDCDeviceCodeAuthorisation:
             serialized_protocol,
             IdentityVerificationMethod.OIDC_DEVICE_CODE.value,
             mock_encrypted_task_request.serialize.return_value,
+            opt_project_id,
         )
         # Check that output value is serialized TaskRequestMessage
         assert trm == mock_task_request_message.serialize.return_value
