@@ -3,11 +3,11 @@ import logging
 from pathlib import Path
 import re
 import sys
-from typing import Dict, List, Optional, cast
+from typing import Dict, List, Optional, Protocol, Union, cast
 from unittest.mock import AsyncMock, MagicMock, Mock, create_autospec
 
 from _pytest.logging import LogCaptureFixture
-from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey
+from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
 import pytest
 from pytest import fixture
 from pytest_mock import MockerFixture
@@ -27,10 +27,11 @@ from bitfount.federated.authorisation_checkers import (
     _SignatureBasedAuthorisation,
 )
 from bitfount.federated.encryption import _RSAEncryption
-from bitfount.federated.exceptions import PodResponseError
+from bitfount.federated.exceptions import PodResponseError, RSAKeyError
+from bitfount.federated.keys_setup import RSAKeyPair
 from bitfount.federated.model_reference import BitfountModelReference
 from bitfount.federated.modeller import _Modeller
-from bitfount.federated.protocols.base import _BaseProtocolFactory
+from bitfount.federated.protocols.base import BaseProtocolFactory
 from bitfount.federated.protocols.model_protocols.federated_averaging import (
     FederatedAveraging,
 )
@@ -69,10 +70,28 @@ PUBLIC_KEY_PATH = TEST_SECURITY_FILES / "test_public.testkey"
 PRIVATE_KEY_PATH = TEST_SECURITY_FILES / "test_private.testkey"
 
 
+class _RegisteredPublicKeyPatcher(Protocol):
+    def __call__(
+        self,
+        key: Optional[Union[RSAPublicKey, RSAPrivateKey]],
+        is_private_key_mock: bool = False,
+    ) -> None:
+        ...
+
+
 @unit_test
 class TestModeller:
     """Unit tests for Modeller."""
 
+    @fixture(params=(True, False), ids=("project_id_incl", "no_project_id"))
+    def opt_project_id(self, request: PytestRequest) -> Optional[str]:
+        """Project ID."""
+        incl_project_id: bool = request.param
+        if incl_project_id:
+            return "this-is-a-project-id"
+        else:
+            return None
+
     @fixture
     def protocol_factory_name(self) -> str:
         """Fake BaseProtocolFactory.name attribute."""
@@ -118,7 +137,7 @@ class TestModeller:
         Is parameterised to return one with and without an aggregator attribute.
         """
         mock_protocol_factory: Mock = create_autospec(
-            _BaseProtocolFactory, instance=True
+            BaseProtocolFactory, instance=True
         )
         mock_protocol_factory.algorithm = Mock()
         dump_return_value = {
@@ -132,12 +151,12 @@ class TestModeller:
 
             if mock_protocol_factory_param == "protocol_with_aggregator":
                 dump_return_value["algorithm"] = {
-                    "class_name": "FederatedModelTraining",
+                    "class_name": "bitfount.FederatedModelTraining",
                     "model": {"class_name": model_name},
                 }
             else:
                 dump_return_value["algorithm"] = {
-                    "class_name": "FederatedModelTraining",
+                    "class_name": "bitfount.FederatedModelTraining",
                     "model": {"class_name": "bitfount.BitfountModelReference"},
                 }
 
@@ -188,11 +207,38 @@ class TestModeller:
         return mock_private_key
 
     @fixture
-    def mock_bitfount_hub(self) -> Mock:
+    def username(self) -> str:
+        """Fake username."""
+        return "fakeUser"
+
+    @fixture
+    def mock_bitfount_hub(self, username: str) -> Mock:
         """A mocked BitfountHub instance."""
         mock_bitfount_hub: Mock = create_autospec(BitfountHub, instance=True)
+        mock_bitfount_hub.username = username
         return mock_bitfount_hub
 
+    @fixture
+    def registered_public_key_patcher(
+        self, mock_bitfount_hub: Mock
+    ) -> _RegisteredPublicKeyPatcher:
+        """Patcher for setting the return value of hub.get_user_public_key()."""
+
+        def _patch_registered_public_key(
+            key: Optional[Union[RSAPublicKey, RSAPrivateKey]],
+            is_private_key_mock: bool = False,
+        ) -> None:
+            if key is None:
+                mock_bitfount_hub.get_user_public_key.return_value = None
+            else:
+                if isinstance(key, RSAPrivateKey) or is_private_key_mock:
+                    public_key = cast(RSAPrivateKey, key).public_key()
+                else:
+                    public_key = key
+                mock_bitfount_hub.get_user_public_key.return_value = public_key
+
+        return _patch_registered_public_key
+
     @fixture
     def pod_identifiers(self) -> List[str]:
         """A list of pod identifiers."""
@@ -252,6 +298,7 @@ class TestModeller:
         mocker: MockerFixture,
         model_name: str,
         modeller: _Modeller,
+        opt_project_id: Optional[str],
         pod_identifiers: List[str],
         protocol_factory_name: str,
     ) -> None:
@@ -275,7 +322,7 @@ class TestModeller:
             expected_task_request_body = SerializedProtocol(
                 class_name=protocol_factory_name,
                 algorithm=SerializedAlgorithm(
-                    class_name="FederatedModelTraining",  # must be an instance of BaseModelAlgorithmFactory # noqa: B950
+                    class_name="bitfount.FederatedModelTraining",  # must be an instance of BaseModelAlgorithmFactory # noqa: B950
                     model=SerializedModel(class_name=f"{model_name}"),
                 ),
                 aggregator=SerializedAggregator(class_name=aggregator_factory_name),
@@ -284,7 +331,7 @@ class TestModeller:
             expected_task_request_body = SerializedProtocol(
                 class_name=protocol_factory_name,
                 algorithm=SerializedAlgorithm(
-                    class_name="FederatedModelTraining",  # must be an instance of BaseModelAlgorithmFactory # noqa: B950
+                    class_name="bitfount.FederatedModelTraining",  # must be an instance of BaseModelAlgorithmFactory # noqa: B950
                     model=SerializedModel(class_name="bitfount.BitfountModelReference"),
                 ),
                 aggregator=SerializedAggregator(class_name=aggregator_factory_name),
@@ -295,7 +342,9 @@ class TestModeller:
                 algorithm=SerializedAlgorithm(class_name=algorithm_factory_name),
             )
 
-        modeller_mailbox = await modeller._send_task_requests(pod_identifiers)
+        modeller_mailbox = await modeller._send_task_requests(
+            pod_identifiers, opt_project_id
+        )
 
         # The mailbox returned from Modeller._send_task_requests() should be the
         # same as the one from ModellerMailbox.send_task_requests().
@@ -306,6 +355,7 @@ class TestModeller:
             pod_public_keys=mock_get_public_keys(),
             message_service=mock_message_service,
             task_request_msg_gen=default_task_request_msg_gen,
+            project_id=opt_project_id,
         )
 
     def test_modeller_init_fails_invalid_verification_method(
@@ -328,26 +378,39 @@ class TestModeller:
                 identity_verification_method=fake_method,
             )
 
-    def test_modeller_init_fails_key_based_verification_but_no_key(
+    def test_modeller_init_generates_keys_if_key_based_verification_but_no_key(
         self,
         mock_bitfount_hub: Mock,
         mock_message_service: Mock,
+        mock_private_key: Mock,
         mock_protocol_factory: Mock,
+        mocker: MockerFixture,
+        registered_public_key_patcher: _RegisteredPublicKeyPatcher,
     ) -> None:
-        """Tests Modeller.__init__ fails if key-based verification method, no key."""
-        with pytest.raises(
-            ValueError,
-            match=re.escape(
-                "Key-based identity verification selected but no private key provided."
-            ),
-        ):
-            _Modeller(
-                protocol=mock_protocol_factory,
-                message_service=mock_message_service,
-                bitfounthub=mock_bitfount_hub,
-                identity_verification_method=IdentityVerificationMethod.KEYS,
-                private_key=None,
-            )
+        """Tests autogenerates keys if key-based verification method but no key."""
+        # Mock out key generation/loading
+        mock_public_key = mock_private_key.public_key()
+        mock__get_modeller_keys: Mock = mocker.patch(
+            "bitfount.federated.modeller._get_modeller_keys",
+            autospec=True,
+            return_value=RSAKeyPair(mock_public_key, mock_private_key),
+        )
+        registered_public_key_patcher(None)
+
+        modeller = _Modeller(
+            protocol=mock_protocol_factory,
+            message_service=mock_message_service,
+            bitfounthub=mock_bitfount_hub,
+            identity_verification_method=IdentityVerificationMethod.KEYS,
+            private_key=None,
+        )
+
+        # Check keys generated/loaded and registered
+        mock__get_modeller_keys.assert_called_once()
+        mock_bitfount_hub.register_user_public_key.assert_called_once_with(
+            mock_public_key
+        )
+        assert modeller._private_key == mock_private_key
 
     def test_modeller_init_warns_non_key_verification_but_key_provided(
         self,
@@ -379,12 +442,16 @@ class TestModeller:
         mock_message_service: Mock,
         mock_protocol_factory: Mock,
         mocker: MockerFixture,
+        registered_public_key_patcher: _RegisteredPublicKeyPatcher,
     ) -> None:
         """Tests Modeller.__init__ loads key from path."""
         # Mock out key loading
         mock_load_private_key = mocker.patch.object(
             _RSAEncryption, "load_private_key", autospec=True
         )
+        registered_public_key_patcher(
+            mock_load_private_key.return_value, is_private_key_mock=True
+        )
 
         # Fake key path
         fake_path = create_autospec(Path, instance=True)
@@ -407,8 +474,10 @@ class TestModeller:
         mock_message_service: Mock,
         mock_private_key: Mock,
         mock_protocol_factory: Mock,
+        registered_public_key_patcher: _RegisteredPublicKeyPatcher,
     ) -> None:
         """Tests Modeller.__init__ uses provided key."""
+        registered_public_key_patcher(mock_private_key, is_private_key_mock=True)
         m = _Modeller(
             protocol=mock_protocol_factory,
             message_service=mock_message_service,
@@ -420,6 +489,49 @@ class TestModeller:
         # Check loading done
         assert m._private_key == mock_private_key
 
+    def test_modeller_init_handles_non_key(
+        self,
+        mock_bitfount_hub: Mock,
+        mock_message_service: Mock,
+        mock_protocol_factory: Mock,
+    ) -> None:
+        """Test error handling if non-private-key is passed to Modeller init."""
+        with pytest.raises(TypeError):
+            _Modeller(
+                protocol=mock_protocol_factory,
+                message_service=mock_message_service,
+                bitfounthub=mock_bitfount_hub,
+                identity_verification_method=IdentityVerificationMethod.KEYS,
+                private_key=Mock(),
+            )
+
+    def test_modeller_init_handles_different_key_registered(
+        self,
+        caplog: LogCaptureFixture,
+        mock_bitfount_hub: Mock,
+        mock_message_service: Mock,
+        mock_private_key: Mock,
+        mock_protocol_factory: Mock,
+        registered_public_key_patcher: _RegisteredPublicKeyPatcher,
+        username: str,
+    ) -> None:
+        """Test error handling if other public key registered on hub."""
+        registered_public_key_patcher(Mock())
+        with pytest.raises(
+            RSAKeyError, match="Registered and stored modeller public keys mismatch"
+        ):
+            _Modeller(
+                protocol=mock_protocol_factory,
+                message_service=mock_message_service,
+                bitfounthub=mock_bitfount_hub,
+                identity_verification_method=IdentityVerificationMethod.KEYS,
+                private_key=mock_private_key,
+            )
+        assert (
+            f"Public key already registered for modeller {username} but does not"
+            f" match local public key." in get_error_logs(caplog)
+        )
+
     async def test__modeller_run(
         self,
         mock_modeller_mailbox: Mock,
@@ -486,14 +598,11 @@ class TestModeller:
             "We recommend normalization of continuous features prior to training."
             in get_warning_logs(caplog)
         )
-        assert (
-            modeller.protocol.algorithm.model.param_clipping  # type: ignore[attr-defined] # Reason: algorithm defined with model # noqa: B950
-            == {
-                "prime_q": 2**61 - 1,
-                "precision": 10**10,
-                "num_workers": 2,
-            }
-        )
+        assert model.param_clipping == {
+            "prime_q": 2**61 - 1,
+            "precision": 10**10,
+            "num_workers": 2,
+        }
 
     def test_modeller_run_secure_aggregation_params_steps(
         self,
@@ -536,7 +645,7 @@ class TestModeller:
             caplog.records[-1].msg != "SecureAggregation in use. "
             "We recommend normalization of continuous features prior to training."
         )
-        assert modeller.protocol.algorithm.model.param_clipping is None  # type: ignore[attr-defined] # Reason: algorithm defined with model # noqa: B950
+        assert model.param_clipping is None
 
     def test_modeller_run_secure_aggregation_params_bfmodel(
         self,
@@ -1005,9 +1114,11 @@ class TestModeller:
         mock_private_key: Mock,
         mock_protocol_factory: Mock,
         mocker: MockerFixture,
+        registered_public_key_patcher: _RegisteredPublicKeyPatcher,
     ) -> None:
         """Test request message generator creation for key-based authentication."""
         # Guarantee private key and key-based on modeller
+        registered_public_key_patcher(mock_private_key, is_private_key_mock=True)
         modeller = _Modeller(
             protocol=mock_protocol_factory,
             message_service=mock_message_service,
@@ -1030,31 +1141,6 @@ class TestModeller:
         # Check returned gen
         assert isinstance(msg_gen, _TaskRequestMessageGenerator)
 
-    def test__get_task_request_msg_gen_key_based_fails_no_key(
-        self,
-        mock_bitfount_hub: Mock,
-        mock_message_service: Mock,
-        mock_protocol_factory: Mock,
-    ) -> None:
-        """Test message generator creation failure for key-based authentication.
-
-        Will fail if no key provided.
-        """
-        with pytest.raises(
-            ValueError,
-            match=re.escape(
-                "Key-based identity verification selected but no private key provided."
-            ),
-        ):
-            # Guarantee NO private key and key-based on modeller
-            _Modeller(
-                protocol=mock_protocol_factory,
-                message_service=mock_message_service,
-                bitfounthub=mock_bitfount_hub,
-                private_key=None,
-                identity_verification_method=IdentityVerificationMethod.KEYS,
-            )
-
     def test__get_task_request_msg_gen_saml_based(
         self,
         mocker: MockerFixture,
@@ -1101,6 +1187,7 @@ class TestModeller:
     def test__serialize(
         self,
         algorithm_factory_name: str,
+        caplog: LogCaptureFixture,
         mock_protocol_factory_param: str,
         mocker: MockerFixture,
         model_name: str,
@@ -1108,9 +1195,8 @@ class TestModeller:
         protocol: str,
     ) -> None:
         """Test serializing of model."""
+        # This is because the protocol that has an algorithm also has a model
         if mock_protocol_factory_param == "protocol_with_aggregator":
-            file_name = Path("file_name")
-
             modeller.protocol.class_name = protocol
             mock_algorithm_factory = create_autospec(
                 _BaseModelAlgorithmFactory, instance=True
@@ -1119,14 +1205,17 @@ class TestModeller:
             mock_algorithm_factory.model = Mock(spec=_BaseModel)
             mock_algorithm_factory.model.class_name = model_name
             modeller.protocol.algorithm = mock_algorithm_factory
+        else:
+            # Explicitly deleting model attribute from Mock object to ensure an
+            # AttributeError is raised when trying to access it. This is necessary
+            # because Mocks allow the accessing of any made-up attribute
+            del modeller.protocol.algorithm.model  # type: ignore[union-attr] # Reason: See above # noqa: B950
 
-            assert isinstance(  # nosec
-                modeller.protocol.algorithm, _BaseModelAlgorithmFactory
-            )
+        # Overriding the `algorithms` property to match the behaviour in the target
+        # object (i.e. returning a list of algorithms)
+        modeller.protocol.algorithms = [modeller.protocol.algorithm]  # type: ignore[misc, list-item] # Reason: See above # noqa: B950
+        modeller._serialize(Path("file_name"))
 
-            modeller._serialize(file_name)
-            # Mypy can't detect that the model is actually a Mock
-            if protocol == "ResultsOnly":
-                modeller.protocol.algorithm.model.serialize.assert_not_called()  # type: ignore[union-attr] # Reason: see above # noqa: B950
-            else:
-                modeller.protocol.algorithm.model.serialize.assert_called_once()  # type: ignore[union-attr] # Reason: see above # noqa: B950
+        # Mypy can't detect that the model is actually a Mock
+        if hasattr(modeller.protocol.algorithm, "model"):
+            modeller.protocol.algorithm.model.serialize.assert_called_once()  # type: ignore[union-attr] # Reason: see above # noqa: B950
