@@ -1,15 +1,19 @@
 """Tests for worker and pod classes."""
 import logging
+import os.path
 from pathlib import Path
+import platform
 import re
+import sqlite3
 import time
-from typing import Any, Dict, List, Optional, Protocol, Type
+from typing import Any, Dict, List, Optional, Protocol, Type, cast
 from unittest.mock import AsyncMock, MagicMock, Mock, NonCallableMock, create_autospec
 
 from _pytest.logging import LogCaptureFixture
 from _pytest.monkeypatch import MonkeyPatch
 from _pytest.tmpdir import TempPathFactory
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
+import numpy as np
 import pandas as pd
 import pytest
 from pytest import fixture
@@ -26,8 +30,8 @@ from bitfount.data.schema import BitfountSchema
 from bitfount.data.utils import DatabaseConnection, _generate_dtypes_hash
 from bitfount.federated.authorisation_checkers import IdentityVerificationMethod
 from bitfount.federated.exceptions import PodNameError
+from bitfount.federated.keys_setup import RSAKeyPair
 from bitfount.federated.pod import Pod, PodRegistrationError
-from bitfount.federated.pod_keys_setup import PodKeys
 from bitfount.federated.pod_response_message import _PodResponseMessage
 from bitfount.federated.pod_vitals import _PodVitals
 from bitfount.federated.task_requests import (
@@ -131,6 +135,7 @@ class TestPod:
         # So the following can be used in dict unpacking
         mock_pod_data_config.data_split.args = dict()
         mock_pod_data_config.datasource_args = dict()
+        mock_pod_data_config.auto_tidy = False
         mock_pod_data_config.ignore_cols = None
         return mock_pod_data_config
 
@@ -168,8 +173,8 @@ class TestPod:
 
     @fixture
     def mock_pod_keys(self) -> DataclassMock:
-        """A dataclass mock of PodKeys."""
-        return create_dataclass_mock(PodKeys)
+        """A dataclass mock of RSAKeyPair for a pod."""
+        return create_dataclass_mock(RSAKeyPair)
 
     @fixture
     def approved_pods(self) -> List[str]:
@@ -384,6 +389,217 @@ class TestPod:
                 approved_pods=approved_pods,
             )
 
+    def test_show_datapoints_with_results_in_db_pod_db_false(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_datasource: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        pod_name: str,
+        username: str,
+    ) -> None:
+        """Tests that if pod_db is false, show_datapoints is set to false as well."""
+        pod = Pod(
+            name=pod_name,
+            datasource=mock_datasource,
+            username=username,
+            data_config=mock_pod_data_config,
+            pod_details_config=mock_pod_details_config,
+            hub=mock_bitfount_hub,
+            message_service=mock_message_service_config,
+            access_manager=mock_access_manager,
+            pod_keys=mock_pod_keys,
+            approved_pods=approved_pods,
+            pod_db=False,
+            show_datapoints_with_results_in_db=True,
+        )
+        assert pod.show_datapoints_with_results_in_db is False
+
+    def test_multi_table_datasource_pod_db_false(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        multi_table_excel_source: ExcelSource,
+        pod_name: str,
+        username: str,
+    ) -> None:
+        """Tests that pod_db is false if datasource is multi-tab;e."""
+        pod = Pod(
+            name=pod_name,
+            datasource=multi_table_excel_source,
+            username=username,
+            data_config=mock_pod_data_config,
+            pod_details_config=mock_pod_details_config,
+            hub=mock_bitfount_hub,
+            message_service=mock_message_service_config,
+            access_manager=mock_access_manager,
+            pod_keys=mock_pod_keys,
+            approved_pods=approved_pods,
+            pod_db=True,
+            show_datapoints_with_results_in_db=True,
+        )
+        assert pod.pod_db is False
+
+    @pytest.mark.skipif(
+        condition=platform.system() == "Windows",
+        reason=(
+            "Only works intermittently on Windows. "
+            "Connection to database not always closed properly,"
+            "leading to PermissionError."
+        ),
+    )
+    def test_pod_db(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_datasource: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        pod_name: str,
+        username: str,
+    ) -> None:
+        """Tests database is created for pod."""
+        if os.path.exists(f"{pod_name}.db"):
+            os.remove(f"{pod_name}.db")
+        ds = create_datasource(classification=True)
+        Pod(
+            name=pod_name,
+            datasource=ds,
+            username=username,
+            data_config=mock_pod_data_config,
+            pod_details_config=mock_pod_details_config,
+            hub=mock_bitfount_hub,
+            message_service=mock_message_service_config,
+            access_manager=mock_access_manager,
+            pod_keys=mock_pod_keys,
+            approved_pods=approved_pods,
+            pod_db=True,
+            show_datapoints_with_results_in_db=True,
+        )
+        assert os.path.exists(f"{pod_name}.db")
+        con = sqlite3.connect(f"{pod_name}.db")
+        existing_data = pd.read_sql('SELECT * FROM "datasource"', con)
+        assert existing_data.shape[0] == ds._data.shape[0]
+        assert (
+            existing_data.shape[1] - 1 == ds._data.shape[1]
+        )  # existing_data has the extra datapoint_hash column
+        con.close()
+        os.remove(f"{pod_name}.db")
+
+    def test_datapoint_hash_in_datasource_raises_error(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        pod_name: str,
+        username: str,
+    ) -> None:
+        """Tests error is raised if 'datapoint_hash' is a column name in datasource."""
+        dataset = create_dataset(classification=True)
+        dataset["datapoint_hash"] = np.nan
+        with pytest.raises(ValueError):
+            Pod(
+                name=pod_name,
+                datasource=DataFrameSource(dataset),
+                username=username,
+                data_config=mock_pod_data_config,
+                pod_details_config=mock_pod_details_config,
+                hub=mock_bitfount_hub,
+                message_service=mock_message_service_config,
+                access_manager=mock_access_manager,
+                pod_keys=mock_pod_keys,
+                approved_pods=approved_pods,
+                pod_db=True,
+                show_datapoints_with_results_in_db=True,
+            )
+
+    @pytest.mark.skipif(
+        condition=platform.system() == "Windows",
+        reason=(
+            "Only works intermittently on Windows. "
+            "Connection to database not always closed properly,"
+            "leading to PermissionError."
+        ),
+    )
+    def test_pod_db_new_datapoints(
+        self,
+        approved_pods: List[str],
+        mock_access_manager: Mock,
+        mock_bitfount_hub: Mock,
+        mock_datasource: Mock,
+        mock_message_service_config: DataclassMock,
+        mock_pod_data_config: DataclassMock,
+        mock_pod_details_config: DataclassMock,
+        mock_pod_keys: DataclassMock,
+        pod_name: str,
+        username: str,
+    ) -> None:
+        """Tests only new datapoint added to db."""
+        if os.path.exists(f"{pod_name}.db"):
+            os.remove(f"{pod_name}.db")
+        dataset = create_dataset(classification=True)
+        dataset.drop("Date", inplace=True, axis=1)
+        ds = DataFrameSource(data=dataset[0:3000])
+        Pod(
+            name=pod_name,
+            datasource=ds,
+            username=username,
+            data_config=mock_pod_data_config,
+            pod_details_config=mock_pod_details_config,
+            hub=mock_bitfount_hub,
+            message_service=mock_message_service_config,
+            access_manager=mock_access_manager,
+            pod_keys=mock_pod_keys,
+            approved_pods=approved_pods,
+            pod_db=True,
+            show_datapoints_with_results_in_db=True,
+        )
+        assert os.path.exists(f"{pod_name}.db")
+        con = sqlite3.connect(f"{pod_name}.db")
+        existing_data = pd.read_sql('SELECT * FROM "datasource"', con)
+        assert existing_data.shape[0] == ds._data.shape[0]
+        assert (
+            existing_data.shape[1] - 1 == ds._data.shape[1]
+        )  # existing_data has the extra datapoint_hash column
+        ds = DataFrameSource(data=dataset)
+        Pod(
+            name=pod_name,
+            datasource=ds,
+            username=username,
+            data_config=mock_pod_data_config,
+            pod_details_config=mock_pod_details_config,
+            hub=mock_bitfount_hub,
+            message_service=mock_message_service_config,
+            access_manager=mock_access_manager,
+            pod_keys=mock_pod_keys,
+            approved_pods=approved_pods,
+            pod_db=True,
+            show_datapoints_with_results_in_db=True,
+        )
+
+        existing_data = pd.read_sql('SELECT * FROM "datasource"', con)
+        # Check that only the new datapoints have been added to the ds
+        assert existing_data.shape[0] == dataset.shape[0]
+        assert existing_data.shape[1] - 1 == dataset.shape[1]
+        con.close()
+        os.remove(f"{pod_name}.db")
+
     def test_schema_raises_error_basesource_flag(
         self,
         approved_pods: List[str],
@@ -630,6 +846,8 @@ class TestPod:
             pod_vitals=mock_pod_vitals,
             pod_identifier=pod.pod_identifier,
             serialized_protocol=task_protocol_details,
+            pod_db=False,
+            show_datapoints_in_results_db=False,
         )
 
     async def test__create_and_run_worker_with_secure_aggregation_unapproved_pods(
@@ -759,6 +977,8 @@ class TestPod:
             pod_vitals=mock_pod_vitals,
             pod_identifier=pod.pod_identifier,
             serialized_protocol=task_protocol_details,
+            pod_db=False,
+            show_datapoints_in_results_db=False,
         )
 
         # Ensure interpod mailbox creation logged
@@ -897,7 +1117,10 @@ class TestPod:
         )
 
         assert authorisation_checker == mock_signature_authorisation.return_value
-        task_protocol_details["algorithm"]["model"].pop("schema")  # type: ignore[typeddict-item] # Reason: testing purposes  # noqa: B950
+        serialized_algorithm = cast(
+            SerializedAlgorithm, task_protocol_details["algorithm"]
+        )
+        serialized_algorithm["model"].pop("schema")
         mock_signature_authorisation.assert_called_once_with(
             pod_response_message=_PodResponseMessage(modeller_name, pod.pod_identifier),
             access_manager=pod._access_manager,
@@ -1594,7 +1817,7 @@ class TestPod:
         executed from a notebook.
         """
         mock_is_notebook = mocker.patch(
-            "bitfount.federated.pod._is_notebook", return_value=is_notebook
+            "bitfount.federated.pod.is_notebook", return_value=is_notebook
         )
         mock_handler = mocker.patch("bitfount.federated.pod._PodVitalsHandler")
         pod._run_pod_vitals_server()
@@ -1739,6 +1962,7 @@ class TestPod:
 
     def test_pod_with_schema_wrong_table_name(
         self,
+        caplog: LogCaptureFixture,
         mock_access_manager: Mock,
         mock_bitfount_hub: Mock,
         mock_datasource: Mock,
@@ -1752,9 +1976,10 @@ class TestPod:
         pod_name: str,
         username: str,
     ) -> None:
-        """Tests schema is re-generated when it is missing the correct table name."""
+        """Tests schema warning is issued when it is missing the correct table name."""
         mock_setup_schema = mocker.patch.object(Pod, "_setup_schema")
         schema = BitfountSchema()
+
         Pod(
             name=pod_name,
             datasource=mock_datasource,
@@ -1767,7 +1992,12 @@ class TestPod:
             access_manager=mock_access_manager,
             pod_keys=mock_pod_keys,
         )
-        mock_setup_schema.assert_called_once()
+
+        mock_setup_schema.assert_not_called()
+        assert (
+            "Provided schema table name does not match to pod name,"
+            " you may need to regenerate the schema." in get_warning_logs(caplog)
+        )
 
     def test_pod_w_schema_auto_tidy(
         self,
