@@ -81,6 +81,16 @@ def aes_key() -> bytes:
     return b"SecretAESKey"
 
 
+@fixture
+def mock_aes_decrypter(mocker: MockerFixture) -> Mock:
+    """Mock AES decryption for cases where message bodies aren't encrypted."""
+    # If the message bodies are not encrypted we need to mock out the decryption
+    # and instead replace it with just returning message bodies.
+    mock_decrypter = mocker.patch.object(_MessageEncryption, "decrypt_incoming_message")
+    mock_decrypter.side_effect = lambda body, key: body
+    return mock_decrypter
+
+
 @fixture
 def mock_message_service() -> Mock:
     """Mock message service."""
@@ -1034,6 +1044,76 @@ class TestWorkerMailbox:
             {"msg": "message"}, _BitfountMessageType.LOG_MESSAGE
         )
 
+    @pytest.mark.parametrize(
+        "process_name",
+        ("aProcessName", None),
+        ids=lambda x: f"process_name_present={bool(x)}",
+    )
+    @pytest.mark.parametrize(
+        "thread_name",
+        ("aThreadName", None),
+        ids=lambda x: f"thread_name_present={bool(x)}",
+    )
+    def test_log_message_handler(
+        self,
+        mock_aes_decrypter: Mock,
+        mocker: MockerFixture,
+        modeller_mailbox_id: str,
+        modeller_name: str,
+        opt_task_id: Optional[str],
+        pod_identifier: str,
+        process_name: Optional[str],
+        thread_name: Optional[str],
+        worker_mailbox: _WorkerMailbox,
+        worker_mailbox_id: str,
+    ) -> None:
+        """Test LOG_MESSAGE handler modifies record as expected."""
+        # Mock out logging framework interaction
+        mock_logger = mocker.patch(
+            "bitfount.federated.transport.worker_transport.logger", autospec=True
+        )
+        mock_logging = mocker.patch(
+            "bitfount.federated.transport.worker_transport.logging", autospec=True
+        )
+
+        # Create mock log message
+        mock_log_message_contents = {
+            "msg": "An important log message",
+            "anotherField": "notToBeChanged",
+            "federated": True,
+        }
+        if process_name:
+            mock_log_message_contents["processName"] = process_name
+        if thread_name:
+            mock_log_message_contents["threadName"] = thread_name
+
+        mock_log_message = _BitfountMessage(
+            message_type=_BitfountMessageType.LOG_MESSAGE,
+            body=msgpack.dumps(mock_log_message_contents),
+            recipient=pod_identifier,
+            recipient_mailbox_id=worker_mailbox_id,
+            sender=modeller_name,
+            sender_mailbox_id=modeller_mailbox_id,
+            task_id=opt_task_id,
+        )
+
+        log_message_handler = worker_mailbox._get_log_message_handler()
+        log_message_handler(mock_log_message)
+
+        # Message contents should have been manipulated and logged out
+        expected_log_message = {
+            "msg": "<FROM MODELLER>: An important log message",
+            "anotherField": "notToBeChanged",
+        }
+        if process_name:
+            expected_log_message["processName"] = f"<{process_name}>"
+        if thread_name:
+            expected_log_message["threadName"] = f"<{thread_name}>"
+        mock_logging.makeLogRecord.assert_called_once_with(expected_log_message)
+        mock_logger.handle.assert_called_once_with(
+            mock_logging.makeLogRecord.return_value
+        )
+
 
 @unit_test
 class TestInterPodWorkerMailbox:
