@@ -4,7 +4,7 @@ import json
 from pathlib import Path
 import pickle
 import re
-from typing import Dict, Final, Optional, Type, Union
+from typing import Dict, Final, Optional, Type, Union, cast
 from unittest.mock import AsyncMock, Mock
 
 import httpx
@@ -684,7 +684,8 @@ class TestSynchronousS3Interactions:
         _upload_data_to_s3(s3_upload_url, s3_upload_fields, json.dumps(data))
 
         # Check request file size as actually uploaded
-        request = responses.calls[0].request
+        first_call: responses.Call = cast(responses.Call, responses.calls[0])
+        request = first_call.request
         file_contents: Optional[bytes] = None
         # Extract file content bytes from the multipart/form-data request body
         mpd = MultipartDecoder(
