@@ -1,4 +1,5 @@
 """Tests deployment config files can be loaded."""
+from _pytest.monkeypatch import MonkeyPatch
 import desert
 from jinja2 import Environment, FileSystemLoader, select_autoescape
 import pytest
@@ -33,8 +34,13 @@ def test_staging_deployment_configs_are_valid(config_file: str) -> None:
 
 @integration_test
 @pytest.mark.parametrize("config_file", ["census_income_config.yaml"])
-def test_production_deployment_configs_are_valid(config_file: str) -> None:
+def test_production_deployment_configs_are_valid(
+    config_file: str, monkeypatch: MonkeyPatch
+) -> None:
     """Test that the production pod deployment configs are valid."""
+    # Ensure bitfount environment is correctly set (or unset in this instance)
+    monkeypatch.delenv("BITFOUNT_ENVIRONMENT", raising=False)
+
     expected_username = "anotherTestUser"
     env = Environment(
         loader=FileSystemLoader(DEPLOYMENTS_DIRECTORY), autoescape=select_autoescape()
