@@ -1,12 +1,11 @@
 """Tests the system end-to-end using real web calls."""
-from functools import partial
 import logging
 import multiprocessing
 from multiprocessing import Queue
 import os
 from pathlib import Path
 import time
-from typing import Any, cast
+from typing import Any, Dict, cast
 from unittest.mock import Mock
 
 from _pytest.monkeypatch import MonkeyPatch
@@ -17,15 +16,13 @@ from bitfount import BitfountSchema, DataStructure
 from bitfount.backends.pytorch.models.models import PyTorchTabularClassifier
 from bitfount.data.datasources.base_source import BaseSource
 from bitfount.data.datasources.csv_source import CSVSource
-from bitfount.runners.config_schemas import ModellerConfig, PodConfig
+from bitfount.runners.config_schemas import AlgorithmConfig, ModellerConfig, PodConfig
 from bitfount.runners.modeller_runner import setup_modeller_from_config
 from bitfount.runners.pod_runner import setup_pod_from_config
 from tests.integration import CONFIG_DIR
 from tests.integration.bitfount_web_interactions import (
-    WebdriverBitfountSession,
     get_bitfount_session,
     grant_proactive_access,
-    oidc_flow,
 )
 from tests.integration.utils import (
     load_modeller_config,
@@ -118,11 +115,10 @@ class TestCensusIncomeEndToEnd:
     def mock_modeller_oidc(self, mocker: MockerFixture) -> Any:
         """Mock out oidc auth flow."""
         oidc_webbrowser_import = (
-            "bitfount.federated.transport.identity_verification"
-            ".oidc.webbrowser.open_new_tab"
+            "bitfount.federated.transport.identity_verification.oidc.webbrowser"
         )
 
-        oidc_patcher = mocker.patch(oidc_webbrowser_import)
+        oidc_patcher: Mock = mocker.patch(oidc_webbrowser_import)
         return oidc_patcher
 
     @pytest.fixture
@@ -148,7 +144,7 @@ class TestCensusIncomeEndToEnd:
             },
         )
 
-    @pytest.mark.skip("[BIT-1669] Fix e2e tests")
+    @pytest.mark.skip("BIT-2581 must be resolved before this test will pass")
     async def test_local_vs_pod(
         self,
         census_income_modeller_config: ModellerConfig,
@@ -167,6 +163,11 @@ class TestCensusIncomeEndToEnd:
             census_income_modeller_config.pods.identifiers = [
                 census_income_pod_config.pod_id
             ]
+            assert census_income_modeller_config.task.data_structure
+            census_income_modeller_config.task.data_structure.table_config.table = (
+                census_income_pod_config.name
+            )
+
             # Mock BitfountSession
             # Override default return_value with side_effect
             # get_bitfount_session() one. side_effect takes precedence
@@ -186,13 +187,6 @@ class TestCensusIncomeEndToEnd:
                 ),
             ]
 
-            mock_modeller_oidc.side_effect = [
-                partial(
-                    oidc_flow,
-                    username=census_income_modeller_config.modeller.username,
-                    password=modeller_pswd,
-                )
-            ]
             # Load pods
             pod = setup_pod_from_config(census_income_pod_config)
 
@@ -218,13 +212,13 @@ class TestCensusIncomeEndToEnd:
                 pod_id=census_income_pod_config.pod_id,
                 role="general_modeller",
                 # This is a WebdriverBitfountSession as we mocked it out
-                pod_session=cast(WebdriverBitfountSession, pod._session),
+                pod_session=pod._session,
             )
 
             # Load modeller
             # We do this after the pods have started, as we need the pods to have
             # published their public keys to the Hub.
-            modeller, pod_identifiers = setup_modeller_from_config(
+            modeller, pod_identifiers, _ = setup_modeller_from_config(
                 census_income_modeller_config
             )
 
@@ -236,9 +230,11 @@ class TestCensusIncomeEndToEnd:
                 args=(
                     modeller,
                     pod_identifiers,
-                    None,
-                    queue,
                 ),
+                kwargs={
+                    "model_out": None,
+                    "queue": queue,
+                },
             )
 
             # Start modeller
@@ -256,7 +252,9 @@ class TestCensusIncomeEndToEnd:
                 modeller_process.exitcode is not None and modeller_process.exitcode <= 0
             )
 
-            pod_results = queue.get()[0]
+            # Results is a dict of pod ID to metrics dict
+            results: Dict[str, Dict[str, float]] = queue.get()
+            pod_results = results[census_income_pod_config.pod_id]
             # Check pod_results exists
             assert pod_results
             # Ensure pods are still running
@@ -270,9 +268,15 @@ class TestCensusIncomeEndToEnd:
             assert pod_process.exitcode is not None and pod_process.exitcode <= 0
 
             schema.unfreeze()
+            # We know the task is getting a single algorithm, so it's safe to cast
+            algorithm = cast(
+                AlgorithmConfig, census_income_modeller_config.task.algorithm
+            )
             datastructure = DataStructure(table=pod.name, target="TARGET")
-            hyperparameters = census_income_modeller_config.task.model.hyperparameters
-            model_name = census_income_modeller_config.task.model.name
+            assert algorithm.model is not None
+            hyperparameters = algorithm.model.hyperparameters
+
+            model_name = algorithm.model.name
             if model_name:
                 assert PyTorchTabularClassifier == eval(model_name)
                 local_model = PyTorchTabularClassifier(
@@ -284,8 +288,11 @@ class TestCensusIncomeEndToEnd:
                 local_results = local_model.fit(data=datasource)
                 if local_results:
                     # Check results are the same
-                    for k, pod_value in pod_results:
-                        assert pod_value == local_results.get(k)
+                    for metric, metric_value in pod_results.items():
+                        if (
+                            local_metric_value := local_results.get(metric)
+                        ) is not None:
+                            assert metric_value == float(local_metric_value)
                 else:
                     raise ValueError("Local results are None")
         finally:
@@ -304,7 +311,6 @@ class TestCensusIncomeEndToEnd:
             except Exception:
                 pass
 
-    @pytest.mark.skip("[BIT-1669] Fix e2e tests")
     async def test_census_income_e2e_results_only(
         self,
         mock_bitfount_session: Mock,
@@ -324,6 +330,7 @@ class TestCensusIncomeEndToEnd:
         try:
             # Load configs and tie together
             tie_together_configs(modeller_config, pod_1_config, pod_2_config)
+
             # Mock BitfountSession
             # Override default return_value with side_effect
             # get_bitfount_session() one. side_effect takes precedence
@@ -371,7 +378,7 @@ class TestCensusIncomeEndToEnd:
                 pod_id=pod_1_config.pod_id,
                 role="general_modeller",
                 # This is a WebdriverBitfountSession as we mocked it out
-                pod_session=cast(WebdriverBitfountSession, pod_1._session),
+                pod_session=pod_1._session,
             )
             logger.info(f"Granting proactive access to pod {pod_2_config.pod_id}...")
             grant_proactive_access(
@@ -379,13 +386,13 @@ class TestCensusIncomeEndToEnd:
                 pod_id=pod_2_config.pod_id,
                 role="general_modeller",
                 # This is a WebdriverBitfountSession as we mocked it out
-                pod_session=cast(WebdriverBitfountSession, pod_2._session),
+                pod_session=pod_2._session,
             )
 
             # Load modeller
             # We do this after the pods have started, as we need the pods to have
             # published their public keys to the Hub.
-            modeller, pod_identifiers = setup_modeller_from_config(modeller_config)
+            modeller, pod_identifiers, _ = setup_modeller_from_config(modeller_config)
             model_out = tmp_path / "model.out"
 
             modeller_process = multiprocessing.Process(
@@ -394,8 +401,10 @@ class TestCensusIncomeEndToEnd:
                 args=(
                     modeller,
                     pod_identifiers,
-                    model_out,
                 ),
+                kwargs={
+                    "model_out": model_out,
+                },
             )
 
             # Start modeller
