@@ -9,8 +9,20 @@ import os
 from pathlib import Path
 from queue import Queue
 import time
-from typing import Any, Callable, Dict, Generator, Iterable, Type, Union, cast
-from unittest.mock import Mock
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Iterable,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
+from unittest.mock import Mock, create_autospec
 
 from _pytest.monkeypatch import MonkeyPatch
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPublicKey
@@ -22,10 +34,11 @@ from pytest_mock import MockerFixture
 from pytorch_lightning.loggers import TensorBoardLogger
 
 import bitfount
-from bitfount import PodRegistrationError
+from bitfount import AuthenticationHandler, PodRegistrationError
 from bitfount.federated.modeller import _Modeller
 from bitfount.federated.transport.base_transport import _run_func_and_listen_to_mailbox
 from bitfount.federated.transport.message_service import _LargeObjectRequestHandler
+from bitfount.federated.types import _PodResponseType
 from bitfount.hub.authentication_flow import BitfountSession
 from bitfount.runners.config_schemas import ModellerConfig, PodConfig
 from bitfount.runners.modeller_runner import setup_modeller_from_config
@@ -61,12 +74,13 @@ class MockBitfountSession(BitfountSession):
 
     def __init__(self, access_token: str, username: str, **kwargs: Any):
         self._faked_access_token = access_token
-        super().__init__(username=username, **kwargs)
+        self._username = username
+        super().__init__(**kwargs)
 
     @property
-    def access_token(self) -> str:
+    def message_service_metadata(self) -> List[Tuple[str, str]]:
         """The supplied access token to provide to the message service."""
-        return self._faked_access_token
+        return [("token", self._faked_access_token)]
 
     def authenticate(self) -> None:
         """Same as BitfountSession but doesn't authenticate anything."""
@@ -83,10 +97,10 @@ class TestEndToEnd:
     """Tests end-to-end functionality using mocks."""
 
     @fixture(autouse=True)
-    def mock_am(
+    def patch_get_access_manager_key(
         self, monkeypatch: MonkeyPatch, access_manager_public_key: RSAPublicKey
     ) -> None:
-        """Mock get_access_manager_key."""
+        """Patch get_access_manager_key."""
         import bitfount.hub.api as api_
 
         monkeypatch.setattr(
@@ -96,8 +110,22 @@ class TestEndToEnd:
         )
 
     @fixture(autouse=True)
-    def mock_logger_save_dir(self, monkeypatch: MonkeyPatch, tmp_path: Path) -> None:
-        """Mock pytorch logger."""
+    def patch_am_signature_checking(self, mocker: MockerFixture) -> Mock:
+        """Patch access manager signature-based authorisation checking.
+
+        Sets "ACCEPT" by default but returns the mock so this can be overridden.
+        """
+        from bitfount.hub.api import BitfountAM
+
+        mock_signature_checker: Mock = mocker.patch.object(
+            BitfountAM, "check_signature_based_access_request", autospec=True
+        )
+        mock_signature_checker.return_value = _PodResponseType.ACCEPT
+        return mock_signature_checker
+
+    @fixture(autouse=True)
+    def patch_logger_save_dir(self, monkeypatch: MonkeyPatch, tmp_path: Path) -> None:
+        """Patch pytorch logger save directory."""
         from bitfount.backends.pytorch.models.base_models import BasePyTorchModel
 
         monkeypatch.setattr(
@@ -158,9 +186,14 @@ class TestEndToEnd:
         def _create_mock_bitfount_session(
             username: str, access_token: str
         ) -> MockBitfountSession:
-            session = MockBitfountSession(access_token=access_token, username=username)
-            session.user_storage_path = tmp_path / username
-            session.token_file = session.user_storage_path / ".token"
+            handler = create_autospec(AuthenticationHandler)
+            handler.user_storage_path = tmp_path / username
+            handler.token_file = handler.user_storage_path / ".token"
+            session = MockBitfountSession(
+                access_token=access_token,
+                username=username,
+                authentication_handler=handler,
+            )
             return session
 
         return _create_mock_bitfount_session
@@ -188,7 +221,7 @@ class TestEndToEnd:
 
             # upload_fields is unused below
             # noinspection PyUnusedLocal
-            def store_large_object(
+            async def store_large_object(
                 upload_url: _S3PresignedPOSTURL,
                 upload_fields: _S3PresignedPOSTFields,
                 large_object: bytes,
@@ -202,7 +235,7 @@ class TestEndToEnd:
                 """
                 large_object_storage[upload_url] = large_object
 
-            def get_large_object(url: _S3PresignedURL) -> bytes:
+            async def get_large_object(url: _S3PresignedURL) -> bytes:
                 """Mock downloading the object from the download URL.
 
                 Args:
@@ -236,6 +269,7 @@ class TestEndToEnd:
     def _modeller_send_task_requests(
         modeller: _Modeller,
         pod_identifiers: Iterable[str],
+        project_id: Optional[str] = None,
     ) -> None:
         """Helper for sending Modeller task requests in a separate process.
 
@@ -246,7 +280,9 @@ class TestEndToEnd:
         # and use asyncio.run() to simulate async def.
         async def _run_modeller() -> None:
             """Asyncronous modeller runner to process and run task requests."""
-            modeller_mailbox = await modeller._send_task_requests(pod_identifiers)
+            modeller_mailbox = await modeller._send_task_requests(
+                pod_identifiers, project_id
+            )
             await _run_func_and_listen_to_mailbox(
                 modeller_mailbox.process_task_request_responses(), modeller_mailbox
             )
@@ -330,7 +366,6 @@ class TestEndToEnd:
 
             assert p.exitcode is not None and p.exitcode <= 0
 
-    @pytest.mark.skip("[BIT-1669] Fix e2e tests")
     @backend_test
     @pytest.mark.parametrize("app", [(False, True, False)], indirect=True)
     @pytest.mark.timeout(STD_TIMEOUT)
@@ -346,9 +381,13 @@ class TestEndToEnd:
         mock_s3_data_upload_in_api_module: Mock,
         mocker: MockerFixture,
         modeller_config: ModellerConfig,
+        patch_am_signature_checking: Mock,
         pod_config: PodConfig,
     ) -> None:
         """Tests Modeller handling if task requests are rejected by pods."""
+        # Make access checking return a reject
+        patch_am_signature_checking.return_value = _PodResponseType.UNAUTHORISED
+
         try:
             with multiprocessing.Manager() as manager:
                 modeller_token = "modeller_token"
@@ -384,7 +423,9 @@ class TestEndToEnd:
                     return_value=pod.schema,
                 )
 
-                modeller, pod_identifiers = setup_modeller_from_config(modeller_config)
+                modeller, pod_identifiers, _ = setup_modeller_from_config(
+                    modeller_config
+                )
                 print(f"{modeller._identity_verification_method}")
 
                 # Start pod, give time to spin up
@@ -414,12 +455,14 @@ class TestEndToEnd:
                 assert "INFO" in caplog_records
                 assert (
                     "Task request from e2e_modeller rejected. "
-                    "Protocol not approved" in caplog_records["INFO"]
+                    "Insufficient permissions for the requested task on this pod."
+                    in caplog_records["INFO"]
                 )
                 assert "ERROR" in caplog_records
                 assert (
                     f"Received rejection from {pod_config.pod_id}. "
-                    f"Protocol not approved" in caplog_records["ERROR"]
+                    f"Insufficient permissions for the requested task on this pod."
+                    in caplog_records["ERROR"]
                 )
                 assert "CRITICAL" not in caplog_records
 
@@ -475,7 +518,6 @@ class TestEndToEnd:
             side_effect=_download,
         )
 
-    @pytest.mark.skip("[BIT-1669] Fix e2e tests")
     @backend_test
     @pytest.mark.timeout(EXT_TIMEOUT)
     @pytest.mark.parametrize("use_local_storage", [True, False])
@@ -560,7 +602,9 @@ class TestEndToEnd:
                 pod1 = setup_pod_from_config(pod1_config)
                 pod2 = setup_pod_from_config(pod2_config)
 
-                modeller, pod_identifiers = setup_modeller_from_config(modeller_config)
+                modeller, pod_identifiers, _ = setup_modeller_from_config(
+                    modeller_config
+                )
                 model_out = tmp_path / "model.out"
 
                 # Start pods and wait to spin up
@@ -583,8 +627,10 @@ class TestEndToEnd:
                     args=(
                         modeller,
                         pod_identifiers,
-                        model_out,
                     ),
+                    kwargs={
+                        "model_out": model_out,
+                    },
                 )
                 modeller_process.start()
 
