@@ -32,14 +32,29 @@ def load_pod_config(config_path: Path, data_path: Path) -> PodConfig:
     # Load base config
     with open(config_path) as f:
         config_yaml = yaml.safe_load(f)
-    config: PodConfig = desert.schema(PodConfig).load(config_yaml)
+
+    config_schema = desert.schema(PodConfig)
+    config_schema.context["config_path"] = config_path
+
+    config: PodConfig = config_schema.load(config_yaml)
 
     suffix = str(int(time.time()))  # unix timestamp
 
-    # Add suffix where needed
+    # Add suffix where needed:
+    # Pod name
     config.name += suffix
+    # Pod display name
     if config.pod_details_config:
         config.pod_details_config.display_name += suffix
+    # Table name in ignore_cols
+    if config.data_config.ignore_cols:
+        updated_ignore_cols = {}
+        for table_name in config.data_config.ignore_cols:
+            updated_ignore_cols[
+                f"{table_name}{suffix}"
+            ] = config.data_config.ignore_cols[table_name]
+        config.data_config.ignore_cols = updated_ignore_cols
+    # Table name in force_stypes
     if config.data_config.force_stypes:
         updated_force_stypes = {}
         for table_name in config.data_config.force_stypes:
@@ -47,6 +62,7 @@ def load_pod_config(config_path: Path, data_path: Path) -> PodConfig:
                 f"{table_name}{suffix}"
             ] = config.data_config.force_stypes[table_name]
         config.data_config.force_stypes = updated_force_stypes
+
     # Update data
     config.data_config.datasource_args["path"] = PathConfig(data_path).path
 
@@ -69,7 +85,11 @@ def load_modeller_config(
     # Load base config
     with open(config_path) as f:
         config_yaml = yaml.safe_load(f)
-    config: ModellerConfig = desert.schema(ModellerConfig).load(config_yaml)
+
+    config_schema = desert.schema(ModellerConfig)
+    config_schema.context["config_path"] = config_path
+
+    config: ModellerConfig = config_schema.load(config_yaml)
 
     if not private_key:
         # Load appropriate private key from secrets onto the config
@@ -141,7 +161,7 @@ def run_modeller_process(
     modeller: _Modeller,
     pod_identifiers: Iterable[str],
     require_all_pods: bool = False,
-    model_out: Path = DEFAULT_MODEL_OUT,
+    model_out: Optional[Path] = DEFAULT_MODEL_OUT,
     queue: Optional[Queue] = None,
 ) -> Optional[Any]:
     """Helper function for sending Modeller requests and running training.
