@@ -7,7 +7,7 @@ from pathlib import Path
 import random
 import string
 from typing import Any, List, Literal, Optional, Protocol, Sequence, Tuple, Union, cast
-from unittest.mock import create_autospec
+from unittest.mock import Mock, create_autospec
 
 from PIL import Image, ImageOps
 from _pytest.logging import LogCaptureFixture
@@ -685,6 +685,10 @@ def create_local_modeller_and_workers(
         workers.append(worker)
 
     # Create modeller
+    mock_hub: Mock = create_autospec(BitfountHub, instance=True)
+    mock_hub.username = modeller_name
+    mock_modeller_private_key: Mock = create_autospec(RSAPrivateKey, instance=True)
+    mock_hub.get_user_public_key.return_value = mock_modeller_private_key.public_key()
     modeller = _Modeller(
         protocol=protocol,
         message_service=LocalMessageService(
@@ -693,9 +697,9 @@ def create_local_modeller_and_workers(
             modeller_mailbox_id=modeller_mailbox_id,
             pod_ids=pod_ids,
         ),
-        bitfounthub=create_autospec(BitfountHub, instance=True),
+        bitfounthub=mock_hub,
         identity_verification_method=IdentityVerificationMethod.KEYS,
-        private_key=create_autospec(RSAPrivateKey, instance=True),
+        private_key=mock_modeller_private_key,
     )
 
     return modeller, workers
